function [clusrand] = clusterrandanalysis(cfg,varargin);

% CLUSTERRANDANALYSIS
%
% Use as
%   [clusrand] = clusterrandanalysis(cfg, data)
% where the configuration can contain
%
% 1. Options for data selection and averaging over selected dimensions
% --------------------------------------------------------------------
%
%   cfg.channel          = cell-array of size Nx1, see CHANNELSELECTION, or
%   cfg.latency          = [begin end], can be 'all' (default)
%   cfg.frequency        = [begin end], can be 'all' (default)
%   cfg.avgoverchan      = 'yes' or 'no' (default)
%   cfg.avgovertime      = 'yes' or 'no' (default)
%   cfg.avgoverfreq      = 'yes' or 'no' (default)
%
% 2. Statistics options
% ---------------------
%
% cfg.statistic       = 'indepsamplesT' (independent samples T-statistic) 
%                     | 'indepsamplesregrT' (independent samples regression coefficient T-statistic)
%                     | 'depsamplesregrT' (dependent samples regression coefficient T-statistic) 
%                     | 'depsamplesT' (dependent samples T-statistic)
%                     | 'actvsblT' (activation versus baseline T-statistic)
%                     | 'indepsamplesF' (independent samples F-statistic)
%                     | 'depsamplesF' (dependent samples F-statistic)
% cfg.alpha           = alpha level of the randomization test
% cfg.clusterteststat = 'maxsum' (default) or 'orderedsums'
% cfg.smallestcluster = smallest cluster size that is large enough to be considered (only
%                       relevant when clusterteststat='orderedsums')
%
% For every statistic, we now list (a) the constraints with respect to the 
% data that should be passed as an argument to this function (in varargin) and 
% (b) a number of fields that are relevant for a particular choice of statistic. 
%
% For the independent samples T-statistic:
% Two data sets must be passed. The first data set belongs to condition 1
% and the second one belongs to condition 2.
% cfg.onetwo         = 'onesided_1<2' (H_0:condition_1<condition_2), 
%                      'onesided_2<1' (H_0:condition_2<condition_1), or
%                      'twosided' (H_0:condition_1=condition_2, the
%                      default).
%
% For the independent samples regression coefficient T-statistic:
% When this statistic is chosen, a predictor variable has to be specified.
% This can be done in two ways: (1) as an array in the configuration 
% field cfg.ext, or (2) in de data field data.ext. If the second way of 
% specifying .ext is used, and there are multiple data sets in the argument, 
% then every data set must contain a scalar field data.ext. 
% cfg.onetwo         = 'onesided_B<0' (H_0: regr. coefficient B<0), 
%                      'onesided_B>0' (H_0: regr. coefficient B>0), or
%                      'twosided' (H_0:regr. coefficient B=0, the default).
%
% For the dependent samples regression coefficient T-statistic:
% When this statistic is chosen, a predictor variable has to be specified.
% This can be done in two ways: (1) as an array in the configuration 
% field cfg.ext, or (2) in de data field data.ext. Contrary to the
% INdependent samples regression coefficient T-statistic, its DEpendent
% counterpart requires that multiple data sets are passed in the argument;
% one data set for every level of the predictor variable.
% cfg.onetwo         = 'onesided_B<0' (H_0: regr. coefficient B<0), 
%                      'onesided_B>0' (H_0: regr. coefficient B>0), or
%                      'twosided' (H_0:regr. coefficient B=0, the default).
%
% For the dependent samples T-statistic:
% Two data sets must be passed. The first data set belongs to condition 1
% and the second one belongs to condition 2. The two data sets must have
% the same number of replications (subjects or trials) and the order of
% these replications must be the same in both.
% cfg.onetwo         = 'onesided_1<2' (H_0:condition_1<condition_2), 
%                      'onesided_2<1' (H_0:condition_2<condition_1), or
%                      'twosided' (H_0:condition_1=condition_2, the
%                      default).
%
% For the activation versus baseline T-statistic:
% Two data sets must be passed. The first data set belongs to the
% activation period and the second data set to the baseline period. 
% The two data sets must have the same number of replications 
% (subjects or trials) and the order of these replications must be the
% same. The first data set must have a positive time axis (the
% activation condition) and the second data set a negative time axis (the
% baseline condition). In the statistical comparison, the time axis of the baseline period is
% shifted forward in time such that its first sample corresponds to
% the first sample of the activation period. The activation versus baseline
% T-statistic is a dependent samples T-statistic that compares every sample
% in the activation period with the time average of the baseline period.
% cfg.onetwo         = 'onesided_act<bl' (H_0:act<bl), 'onesided_bl<act' (H_0:bl<act), or
%                      'twosided' (H_0:act=bl, the default).
%
% For the independent samples F-statistic:
% Two or more data sets must be passed. 
% 
% For the dependent samples F-statistic:
% Two or more data sets must be passed. The order of the conditions 
% corresponds to the order of the data sets. The data sets must have
% the same number of replications (subjects or trials) and the order of
% these replications must be the same in both.
% cfg.contrastcoefs  = matrix of contrast coefficients determining the
%                      effect being tested. The number of columns of this
%                      matrix must be equal to the number of conditions (ncond). 
%                      The default is a matrix that specifies the
%                      main effect of the independent variable. This matrix
%                      has size [(ncond-1),ncond]. 
% cfg.allowedperms   = matrix of permutations of conditions that all have the
%                      same probability (given the data) under the null 
%                      hypothesis. The default is all ncond! possible permutations.
% 
%
% 3. Clustering options
% ---------------------
% 
% cfg.makeclusters = 'yes' (default) | 'no'
% cfg.alphathresh  = alpha level of the (channel,timepoint,frequency)-specific 
%                    test statistic that will be used for thresholding 
% cfg.minnbchan    = minimum number of neighbouring channels in which a 
%                    particular time-frequency-specific t-test has to be significant in order
%                    for it to be included in the clustering algorithm (default=0).
%
% You can specify the neighbours of each channel either by providing them as a structured 
% cell array with or by looking at the sensor positions that are present in the data. 
% This is done either with
%   cfg.neighbourdist  = distance, default is 4 cm
% or with the field
%   cfg.neighbours     = definition of neighbours for each channel, see NEIGHBCHANSELECTION
% which should be structured like this:
%    cfg.neighbours{1}.label = 'Fz';
%    cfg.neighbours{1}.neighblabel = {'Cz', 'F3', 'F3A', 'FzA', 'F4A', 'F4'};
%    cfg.neighbours{2}.label = 'Cz';
%    cfg.neighbours{2}.neighblabel = {'Fz', 'F4', 'RT', 'RTP', 'P4', 'Pz', 'P3', 'LTP', 'LT', 'F3'};
%    cfg.neighbours{3}.label = 'Pz';
%    cfg.neighbours{3}.neighblabel = {'Cz', 'P4', 'P4P', 'Oz', 'P3P', 'P3'};
%    etc.
% Note that a channel is not considered to be a neighbour of itself.
% Electrode or gradiometer positions are obtained from the first dataset, or can be specified
% using either one of 
%    cfg.elec     = structure with EEG electrode positions
%    cfg.grad     = structure with MEG gradiometer positions
%    cfg.elecfile = filename containing EEG electrode positions
%    cfg.gradfile = filename containing MEG gradiometer positions
%
% 5. Miscellaneous options
% ------------------------
%
% cfg.nranddraws     = number of draws from the randomization distribution
% cfg.randomseed     = 'yes' (default), 'no' or seed-number

% Copyright (C) 2005, Eric Maris, NICI, University Nijmegen
%
% $Log: clusterrandanalysis.m,v $
% Revision 1.13  2005/12/06 13:18:08  erimar
% (1) Removed all functionality with respect to channel combinations
% (because it was not sufficiently tested for data sets of realistic
% sizes). (2) Improved and extended the handling of external predictor
% variables (for the independent and the dependent samples regression
% coefficient T-statistics).
%
% Revision 1.12  2005/11/28 11:15:09  erimar
% Improved help
%
% Revision 1.11  2005/08/10 15:35:41  roboos
% fixed cfg.elecfile (accidentally called gradfile), thanks to chrfor
%
% Revision 1.10  2005/08/05 12:19:03  erimar
% Correct an error in the handling of the .ext-field for
% cfg.indepsamplesregrT detected by Vladimir Litvak.
%
% Revision 1.9  2005/08/05 07:48:37  roboos
% added support for cfg.elec/grad/elecfile/gradfile
% cleaned up the help
% removed the "defaults set in xxx" help comments, since those subfunctions are not accessible for the end-user
%
% Revision 1.8  2005/04/22 07:44:41  roboos
% added/corrected copyrights, added a Log tag for CVS, converted to unix ascii
%

warning off;
pack;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the defaults
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isfield(cfg, 'neighbours'),          cfg.neighbours = [];            end;
if ~isfield(cfg, 'neighbourdist'),       cfg.neighbourdist = 4;          end;
if ~isfield(cfg, 'original'),            cfg.original = 0;               end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform some checks.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% determine whether a beween or a within-replications design is requested.
if sum(strcmp(cfg.statistic,{'indepsamplesT','indepsamplesregrT','indepsamplesF'}))
    between = 1;
end;
if sum(strcmp(cfg.statistic,{'depsamplesregrT','depsamplesT','actvsblT','depsamplesF'}))
    between = 0;
end;
if ~(exist('between')==1)
    error('Unknown test statistic.');
end;

onedimtest = sum(strcmp(cfg.statistic,{'indepsamplesT','indepsamplesregrT','depsamplesregrT','depsamplesT','actvsblT'}))>0;

Nvarargin = length(varargin);

if strcmp(cfg.statistic,'indepsamplesT') | strcmp(cfg.statistic,'depsamplesT') | strcmp(cfg.statistic,'actvsblT')
    if Nvarargin~=2
        error('Number of data sets is not equal to 2, as is required by the test statistic.');
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Preprocessing, specific for the activation-versus-baseline statistic.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if strcmp(cfg.statistic,'actvsblT')
    for dataindx=1:2
        try, varargin{dataindx}.time = varargin{dataindx}.toi;       end; % for frequency data
    end;
    poststim=find(varargin{1}.time>=0);
    if length(poststim)~=length(varargin{1}.time)
        error('Not all time points in the first data set are in the activation (post-stimulus) period.');
    end;
    prestim=find(varargin{2}.time<=0);
    if length(prestim)~=length(varargin{2}.time)
        error('Not all time points in the second data set are in the baseline (pre-stimulus) period.');
    end;
    if length(prestim)~=length(poststim)
        error('The time axes of the first (activation) and the second (baseline) data set are of different length.');
    end;
    varargin{2}.time=varargin{2}.time + min(varargin{1}.time) - min(varargin{2}.time);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data selection and averaging over selected dimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if between
    cfg.datarepresentation = 'concatenated';
else % a within-replication design    
    cfg.datarepresentation = 'cell-array';
end;

fprintf('Selecting and formatting the data.\n');
[cfg,data]=prepare_timefreq_data(cfg, varargin{1:Nvarargin});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Reshape the data in a format suitable for clusterrandstatistics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if between
    [nrepl,lengthspatialdim,nfreq,ntime]=size(data.biol);
    data.biol=reshape(data.biol,[nrepl,1,lengthspatialdim,nfreq,ntime]);
else   % within-replication conditions
    data.biolcell=data.biol;
    nwcond=length(data.biolcell);
    [firstnrepl,lengthspatialdim,nfreq,ntime]=size(data.biolcell{1});
    data.biol=reshape(data.biolcell{1},[firstnrepl,1,lengthspatialdim,nfreq,ntime]);
    data.biolcell{1}=[];
    for condindx=2:nwcond
        nrepl=size(data.biolcell{condindx},1);
        if nrepl~=firstnrepl
            error('The number of replications in the different conditions are unequal. This is not allowed with a test statistic for a within-replications design.');
        end;
        data.biol=cat(2,data.biol,reshape(data.biolcell{condindx},[nrepl,1,lengthspatialdim,nfreq,ntime]));
        data.biolcell{condindx}=[];
    end;
end;
% add the dimension wcond to the dimord
s = strfind(data.dimord, 'chan');
data.dimord = [data.dimord(1:(s-1)) 'wcond_chan' data.dimord((s+4):end)];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adding the external variable data.ext
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if between || strcmp(cfg.statistic,'depsamplesregrT')
    if Nvarargin>1
        data.ext=zeros(Nvarargin,1);
    else % Nvarargin=1
        if strcmp(cfg.statistic,'indepsamplesregrT')
            data.ext=zeros(nrepl,1);
        elseif strcmp(cfg.statistic,'depsamplesregrT')
            data.ext=zeros(nwcond,1);
        end;
    end;
    if strcmp(cfg.statistic,'indepsamplesregrT') | strcmp(cfg.statistic,'depsamplesregrT')
    % constructing the predictor variable for the dependent and the
    % independent samples regression T-statistic
        if isfield(cfg,'ext')
            if size(cfg.ext,1)==1
                data.ext=cfg.ext'; % take the transpose of the row vector
            else
                data.ext=cfg.ext;
            end;
        else
            if Nvarargin>1
                for argindx=1:Nvarargin
                    if isfield(varargin{argindx},'ext')
                        data.ext(argindx)=varargin{argindx}.ext;
                    else
                        error('Neither the configuration nor the data contain the field "ext", as is required by the test statistic.');
                    end;
                end;
            else
                if isfield(varargin{1},'ext')
                    if size(varargin{1}.ext,1)==1
                        data.ext=varargin{1}.ext';
                    else
                        data.ext=varargin{1}.ext;
                    end;
                else
                    error('Neither the configuration nor the data contain the field "ext", as is required by the test statistic.');
                end;
            end;
        end;
    else % if the statistic is not the independent nor the dependent samples regression T-statistic
        data.ext=data.design(:,3);
    end;
    if strcmp(cfg.statistic,'indepsamplesregrT') && (length(data.ext)~=size(data.biol,1))
        error('The number of entries in the predictor variable is not equal to the number of replications in the data.');
    end;
    if strcmp(cfg.statistic,'depsamplesregrT') && (length(data.ext)~=size(data.biol,2))
        error('The number of entries in the predictor variable is not equal to the number of replications in the data.');
    end;
end;    



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the neighbourhood geometry.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Calculating the neighbourhood structure of the channels.\n');
[cfg,data] = getneighbgeometry(cfg,data,varargin{1});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run clusterrandstatistics.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[clusrand] = clusterrandstatistics(cfg, data);

warning on;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BEGIN SUBFUNCTION GETNEIGHBGEOMETRY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [cfg,data] = getneighbgeometry(cfg,data,firstdataset);
% get the neighbourhood geometry

if isempty(cfg.neighbours) 
    % Add the grad and elec fields.
    swapmemfile;  % initialization
    firstdataset = swapmemfile(firstdataset);
    if     isfield(cfg, 'grad')
        fprintf('Obtaining the gradiometer configuration from the configuration.\n');
        data.grad = cfg.grad;
    elseif isfield(cfg, 'elec')
        fprintf('Obtaining the electrode configuration from the configuration.\n');
        data.elec = cfg.elec;
    elseif isfield(cfg, 'gradfile')
        fprintf('Obtaining the gradiometer configuration from a file.\n');
        data.grad = read_fcdc_elec(cfg.gradfile);
    elseif isfield(cfg, 'elecfile')
        fprintf('Obtaining the electrode configuration from a file.\n');
        data.elec = read_fcdc_elec(cfg.elecfile);
    elseif isfield(firstdataset, 'grad')
        fprintf('Obtaining the gradiometer configuration from the first dataset.\n');
        data.grad = firstdataset.grad;
    elseif isfield(firstdataset, 'elec')
        fprintf('Obtaining the electrode configuration from the first dataset.\n');
        data.elec = firstdataset.elec;
    end
    firstdataset = swapmemfile(firstdataset);
    if ~(isfield(data,'grad') | isfield(data,'elec'))
        error('The first data structure does not contain a grad or an elec field.');
    end;
    cfg.neighbours=compneighbstructfromgradelec(data,cfg.neighbourdist);
end;
    
if (ischar(cfg.neighbours) & filetype(cfg.neighbours, 'matlab'))
    load(cfg.neighbours);
    [checkvec,selvec]=match_str(data.label,geom.label);
    if length(checkvec)~=length(data.label)
        error('For some channels in the data, the neighbourhood geometry file does not contain information.');
    end;
    data.channeighbstructmat=geom.channeighbstructmat(selvec,selvec);
    clear geom;
else  % the neighbourhood geometry is not available in a file
    % compute CHANNEIGHBSTRUCTMAT, which will be used in the clustering algorithm
    data.channeighbstructmat = makechanneighbstructmat(cfg.neighbours,data.label);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BEGIN SUBFUNCTION COMPNEIGHBSTRUCTFROMGRADELEC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [neighbours]=compneighbstructfromgradelec(data,neighbourdist);
    
% compute the neighbourhood geometry from the gradiometer/electrode positions provided in the data    

if isfield(data, 'grad')
    sens = data.grad;
elseif isfield(data, 'elec')
    sens = data.elec;
else
    error('No gradiometer or electrode configuration present in the data');
end
nsensors = length(sens.label);

% compute the distance between all sensors
dist = zeros(nsensors,nsensors);
for i=1:nsensors
    dist(i,:) = sqrt(sum((sens.pnt(1:nsensors,:) - repmat(sens.pnt(i,:), nsensors, 1)).^2,2))';
end;

% find the neighbouring electrodes based on distance, later we have to restrict the neighbouring 
% electrodes to those actually selected in the dataset
channeighbstructmat = (dist<neighbourdist);

% electrode istelf is not a neighbour
channeighbstructmat=channeighbstructmat.*~eye(nsensors);

% construct a structured cell array with all neighbours
neighbours=cell(1,nsensors);
for i=1:nsensors
    neighbours{i}.label       = sens.label{i};
    neighbours{i}.neighblabel = sens.label(find(channeighbstructmat(i,:)));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BEGIN SUBFUNCTION MAKECHANNEIGHBSTRUCTMAT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [channeighbstructmat] = makechanneighbstructmat(neighbours,label);

% MAKECHANNEIGHBSTRUCTMAT makes the makes the matrix containing the channel
% neighbourhood structure.

nchan=length(label);
channeighbstructmat = logical(zeros(nchan,nchan));
for chan=1:length(neighbours)
    [seld] = match_str(label, neighbours{chan}.label);
    [seln] = match_str(label, neighbours{chan}.neighblabel);
    if isempty(seld)
        % this channel was not present in the data
        continue;
    else
        % add the neighbours of this channel to the matrix
        channeighbstructmat(seld, seln) = 1;
    end
end;

