function multiplotER(cfg, varargin)

% MULTIPLOTER makes plots of event-related fields or potentials versus time
% or of oscillatory activity (power or coherence) versus frequency. Multiple
% datasets can be overlayed.  The plots are arranged according to their
% location specified in the LAYOUTFILE.
%
% Use as:
%   multiplotER(cfg, data)
%   multiplotER(cfg, data. data2, ..., dataN)
%
% The data can be an ERP/ERF produced by TIMELOCKANALYSIS, a
% powerspectrum produced by FREQANALYSIS or a coherencespectrum produced
% by FREQDESCRIPTIVES. If you specify multiple datasets they must contain
% the same channels, etc.
% 
% The cfg structure contains all configurations details for the plotting
%
% cfg.layout        = specification of the layout, see below
% cfg.xparam        = field to be plotted on x-axis (default = 'time')
% cfg.zparam        = field to be plotted on y-axis (default = 'avg')
% cfg.xlim          = 'maxmin' or [xmin xmax]       (default = 'maxmin')
% cfg.ylim          = 'maxmin' or [ymin ymax]       (default = 'maxmin')
% cfg.baseline      = 'yes','no' or [time1 time2]   (default = 'no')  
% cfg.comment       = string of text                (default = date + colors)
%                     Add 'comment' to graph (according to COMNT in the layout)
% cfg.axes          = 'yes', 'no'                   (default = 'yes')
%                     Draw x- and y-axes for each graph
% cfg.box           = 'yes', 'no'                   (default = 'no')
%                     Draw a box around each graph       
% cfg.showlabels    = 'yes', 'no'                   (default = 'no')
% cfg.fontsize      = size of font                  (default = 8)
% 
% If you use the output of freqdescriptives as an input and if you want to visualise
% coherence with respect to a reference-channel, you should specify:
%
% cfg.zparam        =  'cohspctrm'
% cfg.cohrefchannel =  name of reference-channel
%
% The layout defines how the channels are arranged and what the size of each
% subplot is. You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure, that will be used for creating a
% layout. If you want to have more fine-grained control over the layout
% of the subplots, you should create your own layout file.
% 
% See also MULTIPLOTTFR, TOPOPLOTER, TOPOPLOTTFR

% The old way to plot the coherence with respect to a reference channel is 
% still supported. Then specify
% cfg.zparam           = 'powspctrm';
% cfg.cohtargetchannel = string with the channel relative to which coherence
%                        is plotted (default is to plot power)

% Copyright (C) 2003, Ole Jensen
% 
% $Log: multiplotER.m,v $
% Revision 1.11  2005/08/18 12:15:41  jansch
% added support to plot subfields
%
% Revision 1.10  2005/05/17 17:50:37  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.9  2005/04/29 12:46:44  roboos
% small change in help
%
% Revision 1.8  2005/04/29 12:40:43  roboos
% cleaned up and updated the help
%
% Revision 1.7  2005/04/12 10:06:57  olejen
% clf added
%
% Revision 1.6  2005/04/06 07:40:56  jansch
% included option cfg.cohrefchannel. updated help.
%
% Revision 1.5  2005/02/07 17:12:00  roboos
% changed handling of layout files (using new function createlayout), now also supports automatic layout creation based on gradiometer/electrode definition in data, updated help, cleaned up indentation
%
% Revision 1.4  2005/01/27 09:31:49  roboos
% applied autoindentation on code, removed many empty lines and spaces,
% replaced layoutfile reading with read_lay, applied doudavs code to all input arguments
% implemented automatic detection of arguments to plot,
% updated help, changed input from p1, p2, p3... to varargin
%
% Revision 1.3  2004/09/24 15:54:54  roboos
% included the suggested improvements by Doug Davidson: added option cfg.cohtargetchannel
% and updated the help
%
% Revision 1.2  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%

clf

% for backward support with old configurations
if isfield(cfg, 'yparam') && ~isfield(cfg, 'zparam')
  warning('please specify cfg.zparam instead of cfg.yparam');
  cfg.zparam = cfg.yparam;
  cfg = rmfield(cfg, 'yparam');
end

if isfield(cfg,'cohtargetchannel'),
  % replace the powspctrm field with the respective coherence
  % implemented by doudav
  for i=1:length(varargin)
    varargin{i} = pwrspctm2cohspctrm(cfg, varargin{i}); 
  end
end;

if ~isfield(cfg,'baseline'),        cfg.baseline   = 'no'; end;
if ~isfield(cfg,'xlim'),            cfg.xlim       = 'maxmin'; end;
if ~isfield(cfg,'ylim'),            cfg.ylim       = 'maxmin'; end;
if ~isfield(cfg,'comment'),         cfg.comment    = strcat([date '\n']); end;
if ~isfield(cfg,'axes'),            cfg.axes       = 'yes'; end;
if ~isfield(cfg,'showlabels'),      cfg.showlabels = 'no'; end;
if ~isfield(cfg,'box'),             cfg.box        = 'no'; end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize   = 8; end;

if ~isfield(cfg,'zparam'),
  % determine the type of data
  if isfield(varargin{1}, 'avg'),       cfg.zparam     = 'avg';       end;
  if isfield(varargin{1}, 'powspctrm'), cfg.zparam     = 'powspctrm'; end;
end

if ~isfield(cfg,'xparam'),          
  % determine the x-axis that matches the type of data
  if isfield(varargin{1}, 'avg'),       cfg.xparam     = 'time'; end;
  if isfield(varargin{1}, 'powspctrm'), cfg.xparam     = 'foi';  end;
end

if any(strcmp(cfg.zparam, {'cohspctrm' 'avg.cohspctrm' 'avg.zcohspctrm' 'avg.icohspctrm'})),
  if ~isfield(cfg,'cohrefchannel'),
    error('no reference channel specified');
  else
    refindx = strmatch(cfg.cohrefchannel, varargin{1}.label);
    if isempty(refindx),
      error('reference channel not present in data');
    end
  end
end

GRAPHCOLOR =['kbrgkywrgbkywrgbkywrgbkyw'];
colorLabels = [];
for k=2:nargin
  if nargin > 2
    colorLabels = [colorLabels inputname(k) '=' GRAPHCOLOR(k) '\n'];
  end
end

xvalues = getfield(varargin{1} ,cfg.xparam);

% add the colors of the different datasets to the comment
cfg.comment = [cfg.comment colorLabels];

if strcmp(cfg.baseline,'yes')
  if min(xvalues) > 0
    BASELINE = [];
  else
    clear BASELINE
    BASELINE(1) = min(xvalues);
    BASELINE(2) = 0;               
  end
elseif strcmp(cfg.baseline,'no')
  BASELINE = [];
else
  BASELINE(1) = cfg.baseline(1);
  BASELINE(2) = cfg.baseline(2);
end

if strcmp(cfg.xlim,'maxmin')
  xmin = min(xvalues);
  xmax = max(xvalues);   
else
  xmin = cfg.xlim(1); 
  xmax = cfg.xlim(2); 
end

% read or create the layout that will be used for plotting
if isfield(cfg, 'layout')
  lay = createlayout(cfg.layout);               % use specified layout
elseif isfield(cfg, 'layoutname')
  lay = createlayout(cfg.layoutname);           % idem, for backward compatibility
elseif isfield(varargin{1}, 'grad')
  lay = createlayout(varargin{1}.grad);         % create layout from gradiometer definition
elseif isfield(varargin{1}, 'elec')
  lay = createlayout(varargin{1}.elec);         % create layout from electrode definition
else
  lay = createlayout('CTF151s.lay');            % revert to the FCDC default
end

% convert the layout to Ole's style of variable names
X      = lay.prj(:,1);
Y      = lay.prj(:,2);
Width  = lay.box(:,1);
Height = lay.box(:,2);
Lbl    = lay.label;

cfg.GraphCol = GRAPHCOLOR(1);

hold on 

for k=2:nargin
  P          = getsubfield(varargin{k-1}, cfg.zparam);                
  Labels     = getfield(varargin{k-1}, 'label');  
  if any(strcmp(cfg.zparam, {'cohspctrm' 'avg.cohspctrm' 'avg.zcohspctrm' 'avg.icohspctrm'}))
    Labelcmb   = varargin{k-1}.labelcmb;
    sel1       = strmatch(cfg.cohrefchannel, Labelcmb(:,2));
    sel2       = strmatch(cfg.cohrefchannel, Labelcmb(:,1));
    P          = [P(sel1,:); zeros(1,size(P,2)); P(sel2,:)];
    data.label = [Labelcmb(sel1,1);varargin{k-1}.label(refindx);Labelcmb(sel2,2)];
  end
  if nargin > 2
    colorLabels = [colorLabels inputname(k) '=' GRAPHCOLOR(k) '\n'];
  end
  
  if ~isempty(BASELINE)
    tidx = find(xvalues >= BASELINE(1) & xvalues <= BASELINE(2)); 
    if ~isempty(tidx)
      Ptmp = P; 
      tmp = mean(Ptmp(:,tidx),2);
      for l=1:size(P,1)
        Ptmp(l,:) = Ptmp(l,:) - tmp(l);
      end
      P(:,:) = Ptmp; 
    end
  end
  
  if strcmp(cfg.ylim,'maxmin')
    l = [];
    for m=1:length(Lbl)
      ltmp = cellstrmatch(Lbl(m),Labels);
      if ~isempty(ltmp)
        l = [l ltmp];
      end
    end
    ymax = max(max(P(l,:)));
    ymin = min(min(P(l,:)));
  else
    ymin = cfg.ylim(1);
    ymax = cfg.ylim(2);
  end
  
  style = GRAPHCOLOR(k);
  for m=1:length(Lbl)
    l = cellstrmatch(Lbl(m),Labels);
    if ~isempty(l)
      plotWnd(xvalues,P(l,:),[xmin xmax],[ymin ymax],X(m),Y(m),Width(m),Height(m),Lbl(m),cfg,style)
    end
  end
end

xlg = get(gca,'Xlim');
ylg = get(gca,'Ylim');

l = cellstrmatch('COMNT',Lbl);
if ~isempty(l)
  text(X(l),Y(l),sprintf(cfg.comment),'Fontsize',cfg.fontsize);
end

l = cellstrmatch('SCALE',Lbl);
if ~isempty(l)
  plotScales( [xmin xmax],[ymin ymax],X(l),Y(l),Width(1),Height(1),cfg)
end

axis tight
axis off
orient landscape
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotScales(xlim,ylim,xpos,ypos,width,height,cfg)
x1 =  xpos;
x2 =  xpos+width;
y1 =  ypos;
y2 =  ypos+width;
plot([xpos xpos+width xpos+width xpos xpos],[ypos ypos ypos+height ypos+height ypos],'b');
if xlim(1) <=  0 && xlim(2) >= 0
  xs =  xpos+width*([0 0]-xlim(1))/(xlim(2)-xlim(1));
  ys =  ypos+height*(ylim-ylim(1))/(ylim(2)-ylim(1));
  plot(xs,ys,'b');
end

if ylim(1) <= 0 && ylim(2) >= 0
  xs =  xpos+width*(xlim-xlim(1))/(xlim(2)-xlim(1));
  ys =  ypos+height*([0 0]-ylim(1))/(ylim(2)-ylim(1));
  plot(xs,ys,'b');
end

text( x1,y1,num2str(xlim(1),3),'rotation',90,'HorizontalAlignment','Right','VerticalAlignment','middle','Fontsize',cfg.fontsize);
text( x2,y1,num2str(xlim(2),3),'rotation',90,'HorizontalAlignment','Right','VerticalAlignment','middle','Fontsize',cfg.fontsize);
text( x2,y1,num2str(ylim(1),3),'HorizontalAlignment','Left','VerticalAlignment','bottom','Fontsize',cfg.fontsize);
text( x2,y2,num2str(ylim(2),3),'HorizontalAlignment','Left','VerticalAlignment','bottom','Fontsize',cfg.fontsize);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotWnd(x,y,xlim,ylim,xpos,ypos,width,height,label,cfg,style)

set(gca,'FontSize',cfg.fontsize);

xidx = find(x >= xlim(1) & x <= xlim(2));
%yidx = find(y >= ylim(1) & y <= ylim(2));

x = x(xidx);
y = y(xidx);

y(find(y > ylim(2))) = ylim(2);
y(find(y < ylim(1))) = ylim(1);

xs =  xpos+width*(x-xlim(1))/(xlim(2)-xlim(1));
ys =  ypos+height*(y-ylim(1))/(ylim(2)-ylim(1));
plot(xs,ys,style)

if strcmp(cfg.showlabels,'yes')                 
  text(xpos,ypos+1.0*height,label,'Fontsize',cfg.fontsize)
end

if strcmp(cfg.axes,'yes')
  if xlim(1) <=  0 && xlim(2) >= 0
    xs =  xpos+width*([0 0]-xlim(1))/(xlim(2)-xlim(1));
    ys =  ypos+height*(ylim-ylim(1))/(ylim(2)-ylim(1));
    plot(xs,ys,'k');
    
  end
  if ylim(1) <= 0 && ylim(2) >= 0
    xs =  xpos+width*(xlim-xlim(1))/(xlim(2)-xlim(1));
    ys =  ypos+height*([0 0]-ylim(1))/(ylim(2)-ylim(1));
    plot(xs,ys,'k');
  end
end

if strcmp(cfg.box,'yes')  
  plot([xpos xpos+width xpos+width xpos xpos],[ypos ypos ypos+height ypos+height ypos],'b');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l = cellstrmatch(str,strlist)
l = [];
for k=1:length(strlist)
  if strcmp(char(str),char(strlist(k)))
    l = [l k];
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function z = pwrspctm2cohspctrm(cfg, freq)
% This creates a copy of freq with new entries in the freq.pwrspctrm. The
% TFR corresponding to cfg.cohtargetchannel is set to ones, but all other
% channels are replaced with the coherence TFRs from freq.cohspctrm.
%
% [z] = pwrspctrm2cohspctrm(cfg, freq);
% 
% freq should be organised in a structure as obtained from 
% the freqdescriptives function. 
% 
% doudav; 24.09.04
z = freq;
[a a1] = match_str( cfg.cohtargetchannel, z.label);
[a b1] = match_str( cfg.cohtargetchannel, z.labelcmb(:,1));
[a b2] = match_str( cfg.cohtargetchannel, z.labelcmb(:,2));
[a c1] = match_str( z.labelcmb(b1,2), z.label);
[a c2] = match_str( z.labelcmb(b2,1), z.label);
targets  = [c1; c2];
targets2 = [b1; b2];
z.powspctrm(a1,:,:) = ones(size(z.powspctrm(a1,:,:)));
for i=1:size(targets,1),
  z.powspctrm(targets(i),:,:) = squeeze(z.cohspctrm(targets2(i),:,:));
end;
clear a a1 b1 b2 c1 c2;
