function [filetype, detail] = filetype(filename, desired);

% FILETYPE determines the filetype of many EEG/MEG/MRI data files
% by looking at the name, extension and optionally (part of) its
% contents.
%
% It tries to determine the global type of file (which usually
% corresponds to the manufacturer of the recording system) and the
% particular subtype (e.g. continuous, average).
%
% You can use this function either as
%   [type, detail] = filetype(filename)
% to get a descriptive string with the filetype, or as
%   [bool] = filetype(filename, type)
% to get a boolean value indicating whether the file is of the
% desired type.
%
% The filetypes of the following manufacturers are recognized
%  - CTF
%  - Neuromag
%  - Yokogawa
%  - ASA
%  - EDF
%  - Neuroscan
%  - Curry
%  - Analyse
%  - EEProbe
%  - BrainVision
%  - BESA
%  - MINC
%  - AFNI
%  - Neuralynx
%  - Plexon
%
% See also READ_XXX_YYY where XXX=manufacturer and YYY=subtype

% Copyright (C) 2003-2005 Robert Oostenveld
%
% $Log: filetype.m,v $
% Revision 1.25  2005/12/16 14:02:20  roboos
% added (commented out) line for bham_bdf
%
% Revision 1.24  2005/12/02 08:38:09  roboos
% added biosemi_bdf EEG data format
%
% Revision 1.23  2005/11/30 11:37:07  roboos
% added support for ced_son
%
% Revision 1.22  2005/09/29 00:48:01  roboos
% dropped *.sqd and changed from yokogawa_sqd to yokogawa_ave (as suggested by Masahiro Shimogawara)
% added support for MBFYS tri and ama files
%
% Revision 1.21  2005/10/04 16:52:24  roboos
% added support for MPI Frankfurt *.dap files
% added check for directory in check_header
% moved directory listing in case of directory to the beginning of the function
% some small typo's and help changes
%
% Revision 1.20  2005/09/09 09:13:32  roboos
% added support for neuralynx_dma
%
% Revision 1.19  2005/09/06 12:43:41  roboos
% renamed plextor into plexon (incorrect company name)
%
% Revision 1.18  2005/09/02 16:28:00  roboos
% changed Plexon *.ddt from plexon_nex to plexon_ddt, since they are different formats and not compatible with each other
%
% Revision 1.17  2005/09/01 09:24:01  roboos
% added Yokogawa MEG formats
% added BESA beamfourmer source reconstruction (besa_src)
% prevent type-clash between brainvision_dat and besa_src
%
% Revision 1.16  2005/08/04 07:43:49  roboos
% added neuralynx log file
%
% Revision 1.15  2005/07/29 11:27:11  roboos
% added BESA tfc, mul and gen
%
% Revision 1.14  2005/05/18 06:49:52  roboos
% changed single & to double &&
%
% Revision 1.13  2005/05/12 07:22:40  roboos
% added Neuralynx filetypes and dataset directory
%
% Revision 1.12  2005/04/27 06:18:57  roboos
% added support for MEG42RS as a valid CTF res4 subformat
%
% Revision 1.11  2005/04/08 06:47:44  roboos
% added BESA paradigm file *.pdg
%
% Revision 1.10  2005/02/16 08:06:32  roboos
% changed behavior of magic number check: if file cannot be read, give warning (instead of error) and indicate false
% changed indentation
%
% Revision 1.9  2005/02/11 07:53:35  roboos
% iadded support for Curry, Polhemus, Plexon and  Besa *.eps
%
% Revision 1.8  2005/02/07 19:26:19  roboos
% added Ole Jensen's *.lay fileformat
%
% Revision 1.7  2004/10/28 09:42:19  roboos
% added AFNI brick and head (based on extension)
%
% Revision 1.6  2004/08/26 15:57:09  roboos
% changed file extension check into case insensitive
% added support for brainvision_marker
%
% Revision 1.5  2004/08/26 11:35:30  roboos
% added MINC based on mnc extension
%
% Revision 1.4  2004/03/29 15:14:30  roberto
% added support for *.dat as brainvision_dat (exported from BVA)
%
% Revision 1.3  2004/03/23 14:46:13  roberto
% fixed bug in eeprobe average, made besa average smarter
%
% Revision 1.2  2004/03/10 14:06:41  roberto
% added Neuromag filetypes, updated and improved help
%
% Revision 1.1  2003/12/05 10:59:43  roberto
% initial submission
%

if nargin<2
  desired = [];
end

% start with unknown values
filetype     = 'unknown';
manufacturer = 'unknown';
content      = 'unknown';

[p, f, x] = fileparts(filename);

if isdir(filename)
  % the directory listing is needed below
  ls = dir(filename);
end

% known CTF file types
if check_extension(filename, '.ds') && isdir(filename)
  filetype = 'ctf_ds';
  manufacturer = 'CTF';
  content = 'MEG dataset';
elseif check_extension(filename, '.res4') && (check_header(filename, 'MEG41RS') | check_header(filename, 'MEG42RS') | check_header(filename, 'MEG4RES'))
  filetype = 'ctf_res4';
  manufacturer = 'CTF';
  content = 'MEG/EEG header information';
elseif check_extension(filename, '.meg4') && check_header(filename, 'MEG41CP')
  filetype = 'ctf_meg4';
  manufacturer = 'CTF';
  content = 'MEG/EEG';
elseif check_extension(filename, '.mrk') && check_header(filename, 'PATH OF DATASET:')
  filetype = 'ctf_mrk';
  manufacturer = 'CTF';
  content = 'marker file';
elseif check_extension(filename, '.mri') && check_header(filename, 'CTF_MRI_FORMAT VER 2.2')
  filetype = 'ctf_mri';
  manufacturer = 'CTF';
  content = 'MRI';
elseif check_extension(filename, '.hdm')
  filetype = 'ctf_hdm';
  manufacturer = 'CTF';
  content = 'volume conduction model';
elseif check_extension(filename, '.hc')
  filetype = 'ctf_hc';
  manufacturer = 'CTF';
  content = 'headcoil locations';
elseif check_extension(filename, '.shape')
  filetype = 'ctf_shape';
  manufacturer = 'CTF';
  content = 'headshape points';
elseif check_extension(filename, '.shape_info')
  filetype = 'ctf_shapeinfo';
  manufacturer = 'CTF';
  content = 'headshape information';

  % known EEProbe file types
elseif check_extension(filename, '.cnt') && check_header(filename, 'RIFF')
  filetype = 'eep_cnt';
  manufacturer = 'EEProbe';
  content = 'EEG';
elseif check_extension(filename, '.avr') && check_header(filename, char([38 0 16 0]))
  filetype = 'eep_avr';
  manufacturer = 'EEProbe';
  content = 'ERP';
elseif check_extension(filename, '.trg')
  filetype = 'eep_trg';
  manufacturer = 'EEProbe';
  content = 'trigger information';
elseif check_extension(filename, '.rej')
  filetype = 'eep_rej';
  manufacturer = 'EEProbe';
  content = 'rejection marks';

  % known ASA file types
elseif check_extension(filename, '.elc')
  filetype = 'asa_elc';
  manufacturer = 'ASA';
  content = 'electrode positions';
elseif check_extension(filename, '.vol')
  filetype = 'asa_vol';
  manufacturer = 'ASA';
  content = 'volume conduction model';
elseif check_extension(filename, '.bnd')
  filetype = 'asa_bnd';
  manufacturer = 'ASA';
  content = 'boundary element model details';
elseif check_extension(filename, '.msm')
  filetype = 'asa_msm';
  manufacturer = 'ASA';
  content = 'ERP';
elseif check_extension(filename, '.msr')
  filetype = 'asa_msr';
  manufacturer = 'ASA';
  content = 'ERP';
elseif check_extension(filename, '.dip')
  % FIXME, can also be CTF dipole file
  filetype = 'asa_dip';
  manufacturer = 'ASA';
elseif check_extension(filename, '.mri')
  % FIXME, can also be CTF mri file
  filetype = 'asa_mri';
  manufacturer = 'ASA';
  content = 'MRI image header';
elseif check_extension(filename, '.iso')
  filetype = 'asa_iso';
  manufacturer = 'ASA';
  content = 'MRI image data';

  % known Neuroscan file types
elseif check_extension(filename, '.avg') && check_header(filename, 'Version 3.0')
  filetype = 'ns_avg';
  manufacturer = 'Neuroscan';
  content = 'averaged EEG';
elseif check_extension(filename, '.cnt') && check_header(filename, 'Version 3.0')
  filetype = 'ns_cnt';
  manufacturer = 'Neuroscan';
  content = 'continuous EEG';
elseif check_extension(filename, '.eeg') && check_header(filename, 'Version 3.0')
  filetype = 'ns_eeg';
  manufacturer = 'Neuroscan';
  content = 'epoched EEG';

  % known Analyze & SPM file types
elseif check_extension(filename, '.hdr')
  filetype = 'analyze_hdr';
  manufacturer = 'Mayo Analyze';
  content = 'PET/MRI image header';
elseif check_extension(filename, '.img')
  filetype = 'analyze_img';
  manufacturer = 'Mayo Analyze';
  content = 'PET/MRI image data';
elseif check_extension(filename, '.mnc')
  filetype = 'minc';
  content = 'MRI image data';

  % known AFNI file types
elseif check_extension(filename, '.brik') | check_extension(filename, '.BRIK')
  filetype = 'afni_brik';
  content = 'MRI image data';
elseif check_extension(filename, '.head') | check_extension(filename, '.HEAD')
  filetype = 'afni_head';
  content = 'MRI header data';

  % known BrainVison file types
elseif check_extension(filename, '.vhdr')
  filetype = 'brainvision_vhdr';
  manufacturer = 'BrainProducts';
  content = 'EEG header';
elseif check_extension(filename, '.vmrk')
  filetype = 'brainvision_vmrk';
  manufacturer = 'BrainProducts';
  content = 'EEG markers';
elseif check_extension(filename, '.vabs')
  filetype = 'brainvision_vabs';
  manufacturer = 'BrainProducts';
  content = 'Brain Vison Analyzer macro';
elseif check_extension(filename, '.eeg')
  % FIXME, can also be Neuroscan epoched EEG data
  filetype = 'brainvision_eeg';
  manufacturer = 'BrainProducts';
  content = 'continuous EEG data';
elseif check_extension(filename, '.seg')
  filetype = 'brainvision_seg';
  manufacturer = 'BrainProducts';
  content = 'segmented EEG data';
elseif check_extension(filename, '.dat') && ~check_header(filename, 'BESA_SA_IMAGE')
  % WARNING this is a very general name, it could be exported BrainVision
  % data but also a BESA beamformer source reconstruction
  filetype = 'brainvision_dat';
  manufacturer = 'BrainProducts';
  content = 'exported EEG data';
elseif check_extension(filename, '.marker')
  filetype = 'brainvision_marker';
  manufacturer = 'BrainProducts';
  content = 'rejection markers';

  % known Polhemus file types
elseif check_extension(filename, '.pos')
  filetype = 'polhemus_pos';
  manufacturer = 'BrainProducts/CTF/Polhemus?'; % actually I don't know whose software it is
  content = 'electrode positions';

  % known Neuralynx file types
elseif check_extension(filename, '.ncs')  && check_header(filename, '####')
  filetype = 'neuralynx_ncs';
  manufacturer = 'Neuralynx';
  content = 'continuously sampled electrophysiological data';
elseif check_extension(filename, '.nev')
  filetype = 'neuralynx_nev';
  manufacturer = 'Neuralynx';
  content = 'event information';
elseif check_extension(filename, '.ntt')
  filetype = 'neuralynx_ntt';
  manufacturer = 'Neuralynx';
  content = 'unknown';
elseif strcmp(lower(f), 'logfile') && strcmp(lower(x), '.txt')
  filetype = 'neuralynx_log';
  manufacturer = 'Neuralynx';
  content = 'log information';
elseif ~isempty(strfind(lower(f), 'dma')) && strcmp(lower(x), '.log')  % this is not a very strong detection
  filetype = 'neuralynx_dma';
  manufacturer = 'Neuralynx';
  content = 'raw data';
elseif isdir(filename) && ~isempty(strmatch('Events.Nev', {ls.name}))
  filetype = 'neuralynx_ds';
  manufacturer = 'Neuralynx';
  content = 'dataset';

  % known Plexon file types
elseif check_extension(filename, '.nex')  && check_header(filename, 'NEX1')
  filetype = 'plexon_nex';
  manufacturer = 'Plexon';
  content = 'electrophysiological data';
elseif check_extension(filename, '.plx')  && check_header(filename, 'PLEX')
  filetype = 'plexon_nex';
  manufacturer = 'Plexon';
  content = 'electrophysiological data';
elseif check_extension(filename, '.ddt')
  filetype = 'plexon_ddt';
  manufacturer = 'Plexon';

  % known Cambridge Electronic Design file types
elseif check_extension(filename, '.smr')
  filetype = 'ced_son';
  manufacturer = 'Cambridge Electronic Design';
  content = 'Spike2 SON filing system';

  % known Neuromag file types
elseif check_extension(filename, '.fif')
  filetype = 'neuromag_fif';
  manufacturer = 'Neuromag';
  content = 'MEG header and data';
elseif check_extension(filename, '.bdip')
  filetype = 'neuromag_bdip';
  manufacturer = 'Neuromag';
  content = 'dipole model';

  % known Yokogawa file types
elseif check_extension(filename, '.ave')
  filetype = 'yokogawa_ave';
  manufacturer = 'Yokogawa';
  content = 'averaged MEG data';
elseif check_extension(filename, '.con')
  filetype = 'yokogawa_con';
  manufacturer = 'Yokogawa';
  content = 'continuous MEG data';
elseif check_extension(filename, '.raw')
  filetype = 'yokogawa_raw';
  manufacturer = 'Yokogawa';
  content = 'evoked/trialbased MEG data';
elseif check_extension(filename, '.mri') && check_header(filename, '####')  % FIXME, not correct
  filetype = 'yokogawa_mri';
  manufacturer = 'Yokogawa';
  content = 'anatomical MRI';

  % known BESA file types
elseif check_extension(filename, '.avr') && strcmp(filetype, 'unknown')
  filetype = 'besa_avr';  % FIXME, can also be EEProbe average EEG
  manufacturer = 'BESA';
  content = 'average EEG';
elseif check_extension(filename, '.elp')
  filetype = 'besa_elp';
  manufacturer = 'BESA';
  content = 'electrode positions';
elseif check_extension(filename, '.eps')
  filetype = 'besa_eps';
  manufacturer = 'BESA';
  content = 'digitizer information';
elseif check_extension(filename, '.sfp')
  filetype = 'besa_sfp';
  manufacturer = 'BESA';
  content = 'sensor positions';
elseif check_extension(filename, '.ela')
  filetype = 'besa_ela';
  manufacturer = 'BESA';
  content = 'sensor information';
elseif check_extension(filename, '.pdg')
  filetype = 'besa_pdg';
  manufacturer = 'BESA';
  content = 'paradigm file';
elseif check_extension(filename, '.tfc')
  filetype = 'besa_tfc';
  manufacturer = 'BESA';
  content = 'time frequency coherence';
elseif check_extension(filename, '.mul')
  filetype = 'besa_mul';
  manufacturer = 'BESA';
  content = 'multiplexed ascii format';
elseif check_extension(filename, '.gen')
  filetype = 'besa_gen';
  manufacturer = 'BESA';
  content = 'generic ascii format';
elseif check_header(filename, 'BESA_SA_IMAGE')
  filetype = 'besa_src';
  manufacturer = 'BESA';
  content = 'beamformer source reconstruction';
  
  % files from Pascal Fries' PhD research at the MPI
elseif check_extension(filename, '.dap') && check_header(filename, char(1))
  filetype = 'mpi_dap';
  manufacturer = 'MPI Frankfurt';
  content = 'electrophysiological data';
elseif isdir(filename) && ~isempty(cell2mat(regexp({ls.name}, '.dap$')))
  filetype = 'mpi_ds';
  manufacturer = 'MPI Frankfurt';
  content = 'electrophysiological data';
  
  % known Curry V4 file types
elseif check_extension(filename, '.dap')
  filetype = 'curry_dap';   % FIXME, can also be MPI Frankfurt electrophysiological data
  manufacturer = 'Curry';
  content = 'data parameter file';
elseif check_extension(filename, '.dat')
  filetype = 'curry_dat';
  manufacturer = 'Curry';
  content = 'raw data file';
elseif check_extension(filename, '.rs4')
  filetype = 'curry_rs4';
  manufacturer = 'Curry';
  content = 'sensor geometry file';
elseif check_extension(filename, '.par')
  filetype = 'curry_par';
  manufacturer = 'Curry';
  content = 'data or image parameter file';
elseif check_extension(filename, '.bd0') | check_extension(filename, '.bd1') | check_extension(filename, '.bd2') | check_extension(filename, '.bd3') | check_extension(filename, '.bd4') | check_extension(filename, '.bd5') | check_extension(filename, '.bd6') | check_extension(filename, '.bd7') | check_extension(filename, '.bd8') | check_extension(filename, '.bd9')
  filetype = 'curry_bd';
  manufacturer = 'Curry';
  content = 'BEM description file';
elseif check_extension(filename, '.bt0') | check_extension(filename, '.bt1') | check_extension(filename, '.bt2') | check_extension(filename, '.bt3') | check_extension(filename, '.bt4') | check_extension(filename, '.bt5') | check_extension(filename, '.bt6') | check_extension(filename, '.bt7') | check_extension(filename, '.bt8') | check_extension(filename, '.bt9')
  filetype = 'curry_bt';
  manufacturer = 'Curry';
  content = 'BEM transfer matrix file';
elseif check_extension(filename, '.bm0') | check_extension(filename, '.bm1') | check_extension(filename, '.bm2') | check_extension(filename, '.bm3') | check_extension(filename, '.bm4') | check_extension(filename, '.bm5') | check_extension(filename, '.bm6') | check_extension(filename, '.bm7') | check_extension(filename, '.bm8') | check_extension(filename, '.bm9')
  filetype = 'curry_bm';
  manufacturer = 'Curry';
  content = 'BEM full matrix file';
elseif check_extension(filename, '.dig')
  filetype = 'curry_dig';
  manufacturer = 'Curry';
  content = 'digitizer file';

  % known Curry V2 file types
elseif check_extension(filename, '.sp0') | check_extension(filename, '.sp1') | check_extension(filename, '.sp2') | check_extension(filename, '.sp3') | check_extension(filename, '.sp4') | check_extension(filename, '.sp5') | check_extension(filename, '.sp6') | check_extension(filename, '.sp7') | check_extension(filename, '.sp8') | check_extension(filename, '.sp9')
  filetype = 'curry_sp';
  manufacturer = 'Curry';
  content = 'point list';
elseif check_extension(filename, '.s10') | check_extension(filename, '.s11') | check_extension(filename, '.s12') | check_extension(filename, '.s13') | check_extension(filename, '.s14') | check_extension(filename, '.s15') | check_extension(filename, '.s16') | check_extension(filename, '.s17') | check_extension(filename, '.s18') | check_extension(filename, '.s19') | check_extension(filename, '.s20') | check_extension(filename, '.s21') | check_extension(filename, '.s22') | check_extension(filename, '.s23') | check_extension(filename, '.s24') | check_extension(filename, '.s25') | check_extension(filename, '.s26') | check_extension(filename, '.s27') | check_extension(filename, '.s28') | check_extension(filename, '.s29') | check_extension(filename, '.s30') | check_extension(filename, '.s31') | check_extension(filename, '.s32') | check_extension(filename, '.s33') | check_extension(filename, '.s34') | check_extension(filename, '.s35') | check_extension(filename, '.s36') | check_extension(filename, '.s37') | check_extension(filename, '.s38') | check_extension(filename, '.s39')
  filetype = 'curry_s';
  manufacturer = 'Curry';
  content = 'triangle or tetraedra list';
elseif check_extension(filename, '.pom')
  filetype = 'curry_pom';
  manufacturer = 'Curry';
  content = 'anatomical localization file';
elseif check_extension(filename, '.res')
  filetype = 'curry_res';
  manufacturer = 'Curry';
  content = 'functional localization file';

  % known MBFYS file types
elseif check_extension(filename, '.tri')
  filetype = 'mbfys_tri';
  manufacturer = 'MBFYS';
  content = 'triangulated surface';
elseif check_extension(filename, '.ama') && check_header(filename, [0 0 0 10])
  filetype = 'mbfys_ama';
  manufacturer = 'MBFYS';
  content = 'BEM volume conduction model';

  % some other known file types
elseif check_extension(filename, '.lay')
  filetype = 'layout';
  manufacturer = 'Ole Jensen';
  content = 'layout of channels for plotting';
elseif check_extension(filename, '.dcm')
  filetype = 'dicom';
  manufacturer = 'Dicom';
  content = 'image data';
elseif check_extension(filename, '.trl')
  filetype = 'fcdc_trl';
  manufacturer = 'F.C.Donders';
  content = 'trial definitions';
elseif check_header(filename, [255 'BIOSEMI']) % check_extension(filename, '.bdf')
  filetype = 'biosemi_bdf';
%   filetype = 'bham_bdf';
  manufacturer = 'Biosemi Data Format';
  content = 'electrophysiological data';
elseif check_extension(filename, '.edf')
  filetype = 'edf';
  manufacturer = 'European Data Format';
  content = 'electrophysiological data';
elseif check_extension(filename, '.mat')
  filetype = 'matlab';
  manufacturer = 'Matlab';
  content = 'Matlab binary data';
end

if strcmp(filetype, 'unknown')
  warning(sprintf('could not determine filetype of %s', filename));
end

% remember the details
detail.filetype     = filetype;
detail.manufacturer = manufacturer;
detail.content      = content;

if ~isempty(desired)
  % return a boolean value instead of a descriptive string
  filetype = strcmp(filetype, desired);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val] = check_extension(filename, ext)
% perform case insensitive string comparison of the extension
if length(filename)<length(ext)
  val = 0;
else
  val = strcmpi(filename((end-length(ext)+1):end), ext);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val] = check_header(filename, head)
if isdir(filename)
  val = 0;
else
  fid = fopen(filename, 'rb');
  if fid<0
    warning(sprintf('could not open %s', filename));
    val = 0;
  else
    [str, siz] = fread(fid, length(head), 'char');
    fclose(fid);
    if siz~=length(head)
      error('could not read the header from the file');
    end
    val = all(char(str(:))==head(:));
  end
end
return

