function [select] = parameterselection(param, data);

% PARAMETERSELECTION selects the parameters that are present as a volume in the data
% add that have a dimension that is compatible with the specified dimensions of the
% volume, i.e. either as a vector or as a 3D volume.
%
% Use as
%   [select] = parameterselection(param, data)
% where
%   param    cell-array, or single string, can be 'all'
%   data     structure with anatomical or functional data
%   select   returns the selected parameters
%
% If only one parameter is present or selected, it will be returned as
% a single string, otherwise the list of parameters will be eturned in
% a cell-array.

% Copyright (C) 2005, Robert oostenveld
%
% $Log: parameterselection.m,v $
% Revision 1.4  2005/09/29 00:37:57  roboos
% if 'all' is specified, do not replace complete list, but only the 'all' element
% prevent double occurences in parameter list
%
% Revision 1.3  2005/09/12 09:50:46  jansch
% added ref as a parameter
%
% Revision 1.2  2005/08/19 17:26:27  roboos
% added support for 'pow' ->  will be replaced by 'avg.pow' (similar for other volumes)
% added help documentation
%

% it should be a cell-array
if ischar(param)
  param = {param};
end

sel = find(strcmp(param, 'all'));
if ~isempty(sel)
  % default is a list of all known volume parameters
  allparam = {
    'inside'
    'anatomy'
    'mask'
    'zscore'
    'tscore'
    'prob'
    'stat'
    'ref'
    'avg.pow'
    'avg.coh'
    'avg.nai'
    'avg.noise'
    'avg.mom'
    'avg.csd'
    'avg.cov'
    'avg.noisecsd'
    'avg.noisecov'
    'var.pow'
    'var.coh'
    'var.nai'
    'var.noise'
    'sem.pow'
    'sem.coh'
    'sem.nai'
    'sem.noise'
    'avgA.pow'
    'avgA.coh'
    'avgA.nai'
    'avgA.noise'
    'avgB.pow'
    'avgB.coh'
    'avgB.nai'
    'avgB.noise'
    };
  param(sel) = [];                          % remove the 'all'
  param      = [param(:)' allparam(:)'];    % add the list
  param      = unique(param);               % ensure that there are no double entries
end

% add support for common functional parameters like 'pow' and 'coh'
for i=1:length(param)
  if ~issubfield(data, param{i}) && issubfield(data, ['avg.' param{i}])
    % replace the parameter xxx by avg.xxx
    param{i} = ['avg.' param{i}];
  end
end

select = {};
for i=1:length(param)
  if issubfield(data, param{i})
    % the field is present, also check whether the dimension is correct
    tmp = getsubfield(data, param{i});
    dim(1) = size(tmp,1);
    dim(2) = size(tmp,2);
    dim(3) = size(tmp,3);
    if all(dim(:)==data.dim(:)) || prod(dim)==prod(data.dim)
      select{end+1} = param{i}; 
    end
  end
end

if length(select)==0
  % return as empty matrix instead of cell-array
  select = [];
elseif length(select)==1
  % return the selection as a string
  select = select{1};
else
  % return as cell-array with strings
end

