function [axial] = planar2axial(cfg, grad)

% PLANAR2AXIAL converts a synthetic CTF planar gradiometer definition 
% into a magnetometer array or an axial gradiometer array. 
% 
% Use as
%   [axial_grad] = planar2axial(cfg, grad)
%
% Where cfg contains the following configuration details
%   cfg.type     = 'magnetometer' or 'gradiometer' (default)
%   cfg.baseline = number (default is 5 cm)
%
% This function assumes that
%   grad.pnt
%   grad.ori
%   grad.label
% exist and that the first Nlabel channels in pnt and ori can be
% used to compute the position of the bottom and top coils in the axial 
% gradiometer channels.
%
% See also AXIAL2PLANAR

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: planar2axial.m,v $
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.2  2004/04/20 19:53:10  roberto
% fixed some small bugs, removed warning/error at the beginning
%
% Revision 1.1  2004/04/20 19:13:28  roberto
% initial version, not finished since it does not match the current (v1.3) version of axial2planar
%

if ~isfield(cfg, 'type'),      cfg.type = 'gradiometer';   end
if ~isfield(cfg, 'baseline'),  cfg.baseline = 5;           end

for chan=1:size(grad.ori,1)
  % ensure that the original orientation is normalized
  grad.ori(chan,:) = grad.ori(chan,:) ./ norm(grad.ori(chan,:));
end

sel_dH = [];
sel_dV = [];
lab_dH = {};
lab_dV = {};
for chan=1:length(grad.label)
  if findstr(grad.label{chan}, '_dH')
    sel_dH(end+1) = chan;
    lab_dH{end+1} = grad.label{chan}(1:(end-3));
  elseif findstr(grad.label{chan}, '_dV')
    sel_dV(end+1) = chan;
    lab_dV{end+1} = grad.label{chan}(1:(end-3));
  end
end
if length(sel_dH)<1
  error('could not find any planar channel');  
elseif length(sel_dH)~=length(sel_dV)
  error('could not find all corresponding planar channels');
end

% get the position and orientation of each planar channel
pnt = grad.pnt(sel_dH,:);
ori = grad.ori(sel_dH,:);

% get the label of each channel
axial.label = lab_dH;
Nchan = length(axial.label);

% add the bottom coil position and orientation to construct magnetometer array
axial.pnt(1:Nchan,:) = pnt;
axial.ori(1:Nchan,:) = ori;
axial.tra = eye(Nchan,Nchan);

if strcmp(cfg.type, 'gradiometer')
  % add a second coil to form a axial gradiometer
  axial.pnt((1:Nchan)+Nchan,:) = axial.pnt + cfg.baseline * axial.ori;
  axial.ori((1:Nchan)+Nchan,:) = axial.ori;
  axial.tra = [axial.tra -eye(Nchan,Nchan)];
else
  % do not add a second coil, it stays a magnetometer array
end

% remember the units if possible
try
  axial.unit  = grad.unit;
end

% rememeber the configuration of this function call, including its version details
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: planar2axial.m,v 1.1 2004/06/28 08:59:38 roboos Exp $';
axial.cfg      = cfg;
