function [newxgrid, newygrid, newzgrid, newtransform] = transformgrid(xgrid, ygrid, zgrid, transform);

% TRANSFORMGRID compacts an x/y/z-grid description into the accompanying
% homogenous transformation matrix. 
%
% Use as
%   [xgrid, ygrid, zgrid, transform] = transformgrid(xgrid, ygrid, zgrid, transform)
%
% Regardless of the input, the output will be
%   xgrid = 1:Nx
%   ygrid = 1:Ny
%   zgrid = 1:Nz
% and the accompanying homogenous transformation matrix.

% Copyright (C) 2005, Robert Oostenveld
%
% $Log: transformgrid.m,v $
% Revision 1.3  2005/02/16 15:15:05  roboos
% made the check for monotonous increasing grids more robust (using 100*eps)
%
% Revision 1.2  2005/02/08 11:00:13  roboos
% fixed yet another bug in the scaling transformation
%
% Revision 1.1  2005/02/08 08:46:17  roboos
% new implementation, based on code from downsamplevolume except that a bug in that code was found and fixed
%

% check the input arguments
if ~all(size(transform)==[4 4])
  error('incorrect size of homogenous transformation matrix');
elseif ~all(abs(diff(xgrid,2))<=100*eps*max(abs(xgrid)))
  error('xgrid is not monotonously decreasing or increasing');
elseif ~all(abs(diff(ygrid,2))<=100*eps*max(abs(ygrid)))
  error('ygrid is not monotonously decreasing or increasing');
elseif ~all(abs(diff(zgrid,2))<=100*eps*max(abs(zgrid)))
  error('zgrid is not monotonously decreasing or increasing');
end  

Nx = length(xgrid);
Ny = length(ygrid);
Nz = length(zgrid);

xmin = min(xgrid);
ymin = min(ygrid);
zmin = min(zgrid);

xmax = max(xgrid);
ymax = max(ygrid);
zmax = max(zgrid);

% the following three transformations are needed to perform the transformation
% newgrid = t3 * t2 * t1 * oldgrid

% first shift to [0 0 0]
t1 = eye(4);
t1(1,4) = -xmin;
t1(2,4) = -ymin;
t1(3,4) = -zmin;
% then scale to a length equal to the new number of voxels minus one
t2 = eye(4);
t2(1,1) = (Nx-1)/(xmax-xmin);
t2(2,2) = (Ny-1)/(ymax-ymin);
t2(3,3) = (Nz-1)/(zmax-zmin);
% and shift back to [1 1 1]
t3 = eye(4);
t3(1,4) = 1;
t3(2,4) = 1;
t3(3,4) = 1;

% combine all the homogenous transformations in such a way that the new
% transformation behaves identically on the new x/y/zgrid as the old
% transformation did on the old x/y/zgrid

newtransform = transform * inv(t3 * t2 * t1);

% update the definition of the axes
newxgrid = 1:Nx;
newygrid = 1:Ny;
newzgrid = 1:Nz;

% % the following code is for testing only  
% [X, Y, Z] = ndgrid(xgrid, ygrid, zgrid);             xyz = [X(:) Y(:) Z(:)];    xyz(:,4) = 1;
% [X, Y, Z] = ndgrid(newxgrid, newygrid, newzgrid); newxyz = [X(:) Y(:) Z(:)]; newxyz(:,4) = 1;
% % this should be all zeros
% disp(transform * xyz' - newtransform * newxyz');


