function [cfg] = rejectartifact(cfg)

% REJECTARTIFACT does user-specified artifact rejection and returns
% a configuration structure with a modified trial definition. The
% data in all trials is scanned for specific artifacts, according to
% user-specified parameters. If no parameters are specified, default
% values will be used. The new configuration contains the updated
% trials.
%
% You should specify the type of artifacts to be detected with
%   cfg.artfctdef.type = {'xxx', 'yyy'}
% The functions artifact_xxx and artifact_yyy will subsequently be
% used to detect artifacts. Yu can make your own artifact detection
% function. The following artifact detection methods are standard
% available: eog, muscle, jump, manual, threshold, clip and ecg.
% 
% A trial that contains an artifact can be rejected completely or
% partially. In case of partial rejection, a minimum triallength of
% artifact-cleaned 'sub'trials can be specified (in seconds). One can
% also not reject the artifacts, in which case they will only be
% marked in the appropriate output configuration section. This is
% usefull for manual inspection of the artifacts.
%   cfg.artfctdef.reject        = 'complete' (default), 'partial' or 'none'
%   cfg.artfctdef.minaccepttim  = duration in seconds (default is 0.1)
% 
% The detected artifacts can optionally be saved in an EEP-format
% rejection file by specifying
%   cfg.artfctdef.writerej      = filename of new rejection file
%
% See also ARTIFACT_EOG, ARTIFACT_MUSCLE, ARTIFACT_JUMP, ARTIFACT_MANUAL,
% ARTIFACT_THRESHOLD, ARTIFACT_CLIP, ARTIFACT_ECG

% These old configuration options are still supported
%   cfg.rejectmuscle      = 'no' or 'yes'
%   cfg.rejecteog         = 'no' or 'yes'
%   cfg.rejectjump        = 'no' or 'yes'
%   cfg.rejectfile        = string with filename

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: rejectartifact.m,v $
% Revision 1.24  2005/10/05 11:12:03  roboos
% updated documentation
% removed cfg.fsample, instead immediately read header and use hdr.Fs in the code
%
% Revision 1.23  2005/10/04 13:17:03  geekra
% Removed some spelling errors.
%
% Revision 1.22  2005/10/04 13:11:20  geekra
% Added some help about new previously not documented artifact
% rejection methods.
%
% Revision 1.21  2005/07/03 20:15:30  roboos
% replaced eval() by feval() for better support by Matlab compiler
%
% Revision 1.20  2005/06/29 12:46:29  roboos
% the following changes were done on a large number of files at the same time, not all of them may apply to this specific file
% - added try-catch around the inclusion of input data configuration
% - moved cfg.version, cfg.previous and the assignment of the output cfg to the end
% - changed help comments around the configuration handling
% - some changes in whitespace
%
% Revision 1.19  2005/06/17 11:06:29  roboos
% changed handling of cfg.rejectXXX options: these are only for backward
% compatibility support, they are still converted to cfg.artfctdef.type
% but cfg.rejectXXX is now removed after teh conversion. This solves a
% problem of conflicting artifact configuration in case of multiple calls
% to rejectartifact and/or preprocessing.
%
% Revision 1.18  2005/05/17 17:50:38  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.17  2005/02/07 17:20:16  roboos
% added fix for the case where REJECTARTIFACT is called from the command line and the configuration does not specify the sampling freqneucy (cfg.fsample is now read from the header file if needed)
%
% Revision 1.16  2005/02/02 14:24:01  jansch
% added cfg=dataset2files
%
% Revision 1.15  2004/11/24 09:59:56  roboos
% fixed small bug that caused the order of the artifact types to be
% alphabetically sorted, this is not desired if the user (i.e. geekra)
% specifies his own artifact function
%
% Revision 1.14  2004/11/15 09:12:34  roboos
% added check for cfg.trl not being present
%
% Revision 1.13  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%
% Revision 1.12  2004/02/12 10:03:27  roberto
% added check for empty trial definition cfg.trl
%
% Revision 1.11  2004/01/27 09:15:09  roberto
% improved handling of different artifact specifications cfg.rejecteog and cfg.artfctdef.type
%
% Revision 1.10  2004/01/21 16:05:11  roberto
% cleaned up documentation, added link to artifact_xxx for documentation
%
% Revision 1.9  2004/01/06 09:31:24  roberto
% fixed some small bugs after testing of JM
% removed reading of header, fsample is present in cfg
%
% Revision 1.8  2004/01/05 12:00:53  roberto
% updated help according to new cfg.artifact.type cell-array
%
% Revision 1.7  2003/12/18 22:52:33  roberto
% added 'none' as option for reject, which results in artifacts being marked but not
% removed from teh data. This is intended to be used for visual inspection of the
% automatically detected artifacts.
%
% Revision 1.6  2003/12/09 09:46:34  roberto
% function responsibility moved from jansch to roboos
% revamped functionality -> pluggable artifact types
% removed drift and threshold from default types
%
% previous changes to this function (now mostly obsolete) were
% - 2003/10/20 marbau and jansch
%   'sub'complete new implementation of artifact-routines
%   to support visual feedback
% - 2003/08/07 11:38:53 jansch
%   fixed bug in definition of default minacceptnumsmp
% - 2003/06/25 17:49:28  roberto
%   updated help
% - 2003/06/17 12:52:24  jansch
%   added the option to reject jumps in the data
%
% 
% Revision 1.5  2003/06/17 09:10:13  jansch
% fixed bug in automatically creating default rejection parameters
%
% Revision 1.4  2003/06/16 14:26:08  jansch
% fixed bug in automatically creating default rejection parameters
%
% Revision 1.3  2003/06/16 14:20:35  jansch
% fixed bug in indexing of partially rejected trials
% 
% Revision 1.2  2003/06/16 14:17:38  jansch
% original trialdefinition is stored in cfg.trlold
%
% Revision 1.1  2003/05/21 10:14:34  roberto
% renamed all artefact into artifact
%

if 0
  % this code snippet ensures that these functions are included in the
  % documentation as dependencies
  try, dum = artifact_ecg;       end
  try, dum = artifact_eog;       end
  try, dum = artifact_muscle;    end
  try, dum = artifact_jump;      end
  try, dum = artifact_clip;      end
  try, dum = artifact_manual;    end
  try, dum = artifact_threshold; end
end

cfg = dataset2files(cfg);

% set the defaults
if ~isfield(cfg, 'artfctdef'),              cfg.artfctdef        = [];         end
if ~isfield(cfg.artfctdef,'type'),          cfg.artfctdef.type   = {};         end;
if ~isfield(cfg.artfctdef,'reject'),        cfg.artfctdef.reject = 'complete'; end;
if ~isfield(cfg.artfctdef,'minaccepttim')   cfg.artfctdef.minaccepttim = 0.1;  end;

% convert from old-style to new-style configuration
if isfield(cfg,'reject')
  warning('converting from old-style artifact configuration to new-style');
  cfg.artfctdef.reject = cfg.reject;
  cfg = rmfield(cfg, 'reject');
end 

% convert from old-style to new-style configuration
if isfield(cfg.artfctdef,'common')
  warning('converting from old-style artifact configuration to new-style');
  if isfield(cfg.artfctdef.common,'minaccepttim')
    cfg.artfctdef.minaccepttim = cfg.artfctdef.common.minaccepttim;
    cfg.artfctdef = rmfield(cfg.artfctdef, 'common');  
  end
end

% ensure that it is a cell array
if ischar(cfg.artfctdef.type)
  cfg.artfctdef.type = {cfg.artfctdef.type};
end

% ensure that it is a row vector
cfg.artfctdef.type = cfg.artfctdef.type(:)';

% ensure that there are trials that can be scanned for artifacts and/or rejected
if ~isfield(cfg, 'trl') || isempty(cfg.trl)
  warning('no trials were selected, cannot perform artifact detection/rejection');
  return;
end

% read the header of the dataset, needed only for sampling frequency
hdr = read_fcdc_header(cfg.headerfile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add the standard artifact types to the artifact type list
% this is neccessary for backward compatibility
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% support the rejectXXX cfg settings for backward compatibility
if isfield(cfg, 'rejectmuscle') 
  if strcmp(cfg.rejectmuscle,'yes'),
    % this overrules the other setting, add it to the type-list
    cfg.artfctdef.type = [{'muscle'} cfg.artfctdef.type];
  elseif strcmp(cfg.rejectmuscle,'no'),
    % this overrules the other setting, remove it from the type-list
    dum = strmatch('muscle', cfg.artfctdef.type, 'exact');
    if ~isempty(dum)
      warning('conflicting artifact specification, not performing artifact_muscle');
      cfg.artfctdef.type(dum) = [];
    end
  end
  cfg = rmfield(cfg, 'rejectmuscle');
end

% support the rejectXXX cfg settings for backward compatibility
if isfield(cfg, 'rejecteog') 
  if strcmp(cfg.rejecteog,'yes'),
    % this overrules the other setting, add it to the type-list
    cfg.artfctdef.type = [{'eog'} cfg.artfctdef.type];
  elseif strcmp(cfg.rejecteog,'no'),
    % this overrules the other setting, remove it from the type-list
    dum = strmatch('eog', cfg.artfctdef.type, 'exact');
    if ~isempty(dum)
      warning('conflicting artifact specification, not performing artifact_eog');
      cfg.artfctdef.type(dum) = [];
    end
  end
  cfg = rmfield(cfg, 'rejecteog');
end

% support the rejectXXX cfg settings for backward compatibility
if isfield(cfg, 'rejectjump') 
  if strcmp(cfg.rejectjump,'yes'),
    % this overrules the other setting, add it to the type-list
    cfg.artfctdef.type = [{'jump'} cfg.artfctdef.type];
  elseif strcmp(cfg.rejectjump,'no'),
    % this overrules the other setting, remove it from the type-list
    dum = strmatch('jump', cfg.artfctdef.type, 'exact');
    if ~isempty(dum)
      warning('conflicting artifact specification, not performing artifact_jump');
      cfg.artfctdef.type(dum) = [];
    end
  end
  cfg = rmfield(cfg, 'rejectjump');
end

% support the rejectXXX cfg settings for backward compatibility
if isfield(cfg, 'rejectfile') 
  % this is slightly different to the ones above, since rejectfile is either 'no' or contains the filename
  if ~strcmp(cfg.rejectfile,'no'),
    % this overrules the other setting, add it to the type-list
    cfg.artfctdef.type = [{'file'} cfg.artfctdef.type];
  elseif strcmp(cfg.rejectfile,'no'),
    % this overrules the other setting, remove it from the type-list
    dum = strmatch('file', cfg.artfctdef.type, 'exact');
    if ~isempty(dum)
      warning('conflicting artifact specification, not performing artifact_file');
      cfg.artfctdef.type(dum) = [];
    end
  end
end

% prevent double occurences of artifact types, ensure that the order remains the same
[dum, i] = unique(cfg.artfctdef.type);
cfg.artfctdef.type = cfg.artfctdef.type(sort(i));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% call the appropriate function for each of the artifact types
% this will produce a Nx2 matrix with the begin and end sample of artifacts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for type=1:length(cfg.artfctdef.type)
  fprintf('evaluating artifact_%s\n', cfg.artfctdef.type{type});
  [cfg, artfctsmp{type}] = feval(sprintf('artifact_%s', cfg.artfctdef.type{type}), cfg);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% combine all artifacts into a single boolean vector
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rejectall = zeros(1,max(cfg.trl(:,2)));
for type=1:length(cfg.artfctdef.type)
  artifact = artfctsmp{type};   % matrix with begin and end samples of detected artifacts
  if ~isempty(artifact)
    for artlop=1:size(artifact,1)
      rejectall(artifact(artlop,1):artifact(artlop,2)) = 1;
    end  
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% write the rejection to an EEP format file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg.artfctdef, 'writerej') && ~isempty(cfg.artfctdef.writerej)
  fid = fopen(cfg.artfctdef.writerej, 'w');
  if fid<0
    error('could not open rejection file for writing');
  else
    % determine the begin and end of each rejected period (in samples)
    rejectonset = find(diff([0 rejectall])== 1);
    rejectofset = find(diff([rejectall 0])==-1);
    % determine the begin and end of each rejected period (in seconds)
    rejectonset = (rejectonset-1)/hdr.Fs;
    rejectofset = (rejectofset-1)/hdr.Fs;
    for rejlop=1:length(rejectonset)
      fprintf(fid, '%f-%f\n', rejectonset(rejlop), rejectofset(rejlop));
    end
    fclose(fid);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remove the trials that (partially) coincide with a rejection mark
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(cfg.artfctdef.reject, 'partial') || strcmp(cfg.artfctdef.reject, 'complete')
  trl = cfg.trl;
  trialok = [];
  count_complete_reject = 0;
  count_partial_reject  = 0;
  for trial=1:size(trl,1)
    % cut out the part of the rejection-axis corresponding with this trial
    rejecttrial = rejectall(trl(trial,1):trl(trial,2));
    if all(rejecttrial==0)
      % the whole trial is good
      trialok = [trialok; trl(trial,:)];
    elseif all(rejecttrial==1)
      % the whole trial is bad
      count_complete_reject = count_complete_reject + 1;
    elseif any(rejecttrial==1) && strcmp(cfg.artfctdef.reject, 'complete')
      % some part of the trial is bad, reject the whole trial
      count_complete_reject = count_complete_reject + 1;
      continue;
    elseif any(rejecttrial==1) && strcmp(cfg.artfctdef.reject, 'partial')
      % some part of the trial is bad, reject only the bad part
      trialnew = [];
      rejecttrial = [0 not(rejecttrial) 0];
      trialnew(:,1) = find(diff(rejecttrial(:))== 1) + trl(trial,1) - 1;
      trialnew(:,2) = find(diff(rejecttrial(:))==-1) + trl(trial,1) - 2;
      trialnew(:,3) = find(diff(rejecttrial(:))== 1) - 1 + trl(trial,3);
      minacceptnumsmp = round(cfg.artfctdef.minaccepttim .* hdr.Fs);
      trialnew(find(trialnew(:,2)-trialnew(:,1)<minacceptnumsmp),:) = [];
      count_partial_reject = count_partial_reject + 1;
      trialok = [trialok; trialnew];
    end
  end

  fprintf('rejected %d trials completely\n', count_complete_reject);
  fprintf('rejected %d trials partially\n', count_partial_reject);
  cfg.trlold = trl;      % return the original trial definition in the configuration
  cfg.trl    = trialok;  % return the cleaned trial definition in the configuration

else
  fprintf('not rejecting any data, only marking the artifacts\n');
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: rejectartifact.m,v 1.24 2005/10/05 11:12:03 roboos Exp $';

