function singleplotER(cfg, varargin)

% SINGLEPLOTER plots the event-related fields or potentials for one
% sensors. 
%
% Use as:
%   sinlgeplotER(cfg, data)
%   singleplotER(cfg, data. data2, ..., dataN)
%
% The data can be an ERF produced by TIMELOCKANALYSIS, a powerspectrum
% produced by FREQANALYSIS or a coherencespectrum produced by
% FREQDESCRIPTIVES. If you specify multiple datasets they must contain
% the same channels, etc.
% 
% The cfg structure contains all configurations details for the plotting
%
% cfg.layout           = specification of the layout, see below
% cfg.baseline         = 'yes','no' or [time1 time2]   (default = 'no')  
% cfg.xparam           = field to be plotted on x-axis (default = 'time')
% cfg.xlim             = 'maxmin' or [xmin xmax]       (default = 'maxmin')
% cfg.yparam           = field to be plotted on y-axis (default = 'avg')
% cfg.ylim             = 'maxmin' or [ymin ymax]       (default = 'maxmin')
% cfg.fontsize         = size of font                  (default = 8)
% cfg.cohtargetchannel = string with the channel relative to which coherence
%                        is plotted (default is to plot power or amplitude)
%
% The layout defines how the channels are arranged and what the size of each
% subplot is. You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure (common for MEG data, since the header
% of the MEG datafile contains the gradiometer information), that will be
% used for creating a layout. If you want to have more fine-grained control
% over the layout of the subplots, you should create your own layout file.
% 
% See also MULTIPLOTTFR, TOPOPLOTER, TOPOPLOTTFR, SINGLEPLOTER

% Copyright (C) 2003, Ole Jensen
% 
% $Log: singleplotER.m,v $
% Revision 1.3  2005/05/17 17:50:38  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.2  2005/04/17 18:51:05  olejen
% Option colorgraph added
%
% Revision 1.1  2005/04/06 13:59:05  olejen
% Plot EF for a single channel. Modified from multiplotER.m
%
% Revision 1.5  2005/02/07 17:12:00  roboos
% changed handling of layout files (using new function createlayout), now also supports automatic layout creation based on gradiometer/electrode definition in data, updated help, cleaned up indentation
%
% Revision 1.4  2005/01/27 09:31:49  roboos
% applied autoindentation on code, removed many empty lines and spaces,
% replaced layoutfile reading with read_lay, applied doudavs code to all input arguments
% implemented automatic detection of arguments to plot,
% updated help, changed input from p1, p2, p3... to varargin
%
% Revision 1.3  2004/09/24 15:54:54  roboos
% included the suggested improvements by Doug Davidson: added option cfg.cohtargetchannel
% and updated the help
%
% Revision 1.2  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%

if isfield(cfg,'cohtargetchannel'),
  % replace the powspctrm field with the respective coherence
  % implemented by doudav
  for i=1:length(varargin)
    varargin{i} = pwrspctm2cohspctrm(cfg, varargin{i}); 
  end
end;

if ~isfield(cfg,'baseline'),        cfg.baseline   = 'no'; end;
if ~isfield(cfg,'xlim'),            cfg.xlim       = 'maxmin'; end;
if ~isfield(cfg,'ylim'),            cfg.ylim       = 'maxmin'; end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize   = 8; end;
if ~isfield(cfg,'graphcolor')       cfg.graphcolor = ['brgkywrgbkywrgbkywrgbkyw'];end;


GRAPHCOLOR = ['k' cfg.graphcolor ];

if ~isfield(cfg,'yparam'),
  % determine the type of data
  if isfield(varargin{1}, 'avg'),       cfg.yparam     = 'avg';       end;
  if isfield(varargin{1}, 'powspctrm'), cfg.yparam     = 'powspctrm'; end;
end

if ~isfield(cfg,'xparam'),          
  % determine the x-axis that matches the type of data
  if isfield(varargin{1}, 'avg'),       cfg.xparam     = 'time'; end;
  if isfield(varargin{1}, 'powspctrm'), cfg.xparam     = 'foi';  end;
end

%colorLabels = [];
%for k=2:nargin
%  if nargin > 2
%    colorLabels = [colorLabels inputname(k) '=' GRAPHCOLOR(k) '\n'];
%  end
%end

xvalues = getfield(varargin{1} ,cfg.xparam);


if strcmp(cfg.baseline,'yes')
  if min(xvalues) > 0
    BASELINE = [];
  else
    clear BASELINE
    BASELINE(1) = min(xvalues);
    BASELINE(2) = 0;               
  end
elseif strcmp(cfg.baseline,'no')
  BASELINE = [];
else
  BASELINE(1) = cfg.baseline(1);
  BASELINE(2) = cfg.baseline(2);
end

if strcmp(cfg.xlim,'maxmin')
  xmin = min(xvalues);
  xmax = max(xvalues);   
else
  xmin = cfg.xlim(1); 
  xmax = cfg.xlim(2); 
end



% Pick the channel
if isfield(cfg,'channelindex') && isfield(cfg,'channelname')
    warning('You have defined both cfg.channelindex and cfg.channelname; cfg.channelname will be used');
    cfg = rmfield(cfg,'channelindex');
end

if ~isfield(cfg,'channelindex') && ~isfield(cfg,'channelname')
    error('cfg.channelindex or cfg.channelname must be define');
end


if isfield(cfg,'channelname')
    cfg.channelname = channelselection(cfg.channelname,varargin{1}.label);
    if isempty(cfg.channelname)
        error(['Channel ' cfg.channelname ' not found']);
    else
        cfg.channelindex =  match_str(varargin{1}.label, cfg.channelname);
    end
end


% convert the layout to Ole's style of variable names

cfg.GraphCol = GRAPHCOLOR(1);

hold on 
colorLabels = [];
for k=2:nargin
  P = getfield(varargin{k-1}, cfg.yparam);                
  Labels = getfield(varargin{k-1}, 'label');  
  
  if nargin > 2
    colorLabels = [colorLabels inputname(k) '=' GRAPHCOLOR(k) ' '];
  end
  
  if ~isempty(BASELINE)
    tidx = find(xvalues >= BASELINE(1) & xvalues <= BASELINE(2)); 
    if ~isempty(tidx)
      Ptmp = P; 
      tmp = mean(Ptmp(:,tidx),2);
      for l=1:size(P,1)
        Ptmp(l,:) = Ptmp(l,:) - tmp(l);
      end
      P(:,:) = Ptmp; 
    end
  end
  
  if strcmp(cfg.ylim,'maxmin')
    ymax = max(max(P(cfg.channelindex,:)));
    ymin = min(min(P(cfg.channelindex,:)));
  else
    ymin = cfg.ylim(1);
    ymax = cfg.ylim(2);
  end
  
  
  style = GRAPHCOLOR(k);
  plot(xvalues,P(cfg.channelindex,:),style)
  xlim([xmin xmax]);
  ylim([ymin ymax])
end




if ~isfield(cfg,'channelname')
    cfg.channelname = data.label{cfg.channelindex};
end
    
% title([sprintf(colorLabels)])

% title([cfg.channelname ' / ' num2str(cfg.channelindex) ' -' sprintf(colorLabels)])

if isempty(colorLabels)
    title([char(cfg.channelname) ' / ' num2str(cfg.channelindex) ])
else
    title([char(cfg.channelname) ' / ' num2str(cfg.channelindex) ' : ' sprintf(colorLabels)])
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function l = cellstrmatch(str,strlist)
l = [];
for k=1:length(strlist)
  if strcmp(char(str),char(strlist(k)))
    l = [l k];
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function z = pwrspctm2cohspctrm(cfg, freq)
% This creates a copy of freq with new entries in the freq.pwrspctrm. The
% TFR corresponding to cfg.cohtargetchannel is set to ones, but all other
% channels are replaced with the coherence TFRs from freq.cohspctrm.
%
% [z] = pwrspctrm2cohspctrm(cfg, freq);
% 
% freq should be organised in a structure as obtained from 
% the freqdescriptives function. 
% 
% doudav; 24.09.04
z = freq;
[a a1] = match_str( cfg.cohtargetchannel, z.label);
[a b1] = match_str( cfg.cohtargetchannel, z.labelcmb(:,1));
[a b2] = match_str( cfg.cohtargetchannel, z.labelcmb(:,2));
[a c1] = match_str( z.labelcmb(b1,2), z.label);
[a c2] = match_str( z.labelcmb(b2,1), z.label);
targets  = [c1; c2];
targets2 = [b1; b2];
z.powspctrm(a1,:,:) = ones(size(z.powspctrm(a1,:,:)));
for i=1:size(targets,1),
  z.powspctrm(targets(i),:,:) = squeeze(z.cohspctrm(targets2(i),:,:));
end;
clear a a1 b1 b2 c1 c2;
