function [source] = source2full(source);

% SOURCE2FULL recreates the grid locations outside the brain in the source 
% reconstruction, so that the source volume again describes the full grid.
% This undoes the memory savings that can be achieved using SOURC2SPARSE
% and makes it possible again to plot  the source volume and save it to an
% external file.
%
% Use as
%   [source] = source2full(source)
%
% See also SOURCE2SPARSE

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: source2full.m,v $
% Revision 1.14  2005/09/09 17:08:21  jansch
% support for ref as an input-field
%
% Revision 1.13  2005/09/08 06:50:21  jansch
% admit stat as an input field
%
% Revision 1.12  2005/08/23 13:15:16  jansch
% added source.avg.csd and noisecsd
%
% Revision 1.11  2005/06/29 12:46:29  roboos
% the following changes were done on a large number of files at the same time, not all of them may apply to this specific file
% - added try-catch around the inclusion of input data configuration
% - moved cfg.version, cfg.previous and the assignment of the output cfg to the end
% - changed help comments around the configuration handling
% - some changes in whitespace
%
% Revision 1.10  2005/05/17 17:50:38  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.9  2005/03/13 13:03:39  jansch
% added outside nans for var and sem.
%
% Revision 1.8  2005/02/09 13:03:09  roboos
% fixed bug for matlab 70 (mask is logical)
%
% Revision 1.7  2005/01/31 13:49:15  roboos
% fixed bug in sem.nai
%
% Revision 1.6  2005/01/25 11:08:44  roboos
% added var and sem as fields containing source parameters
%
% Revision 1.5  2004/09/23 15:07:16  roboos
% added support for lbex as cell-array in the source structure
%
% Revision 1.4  2004/09/08 08:56:17  jansch
% included correct inside/outside for trialA/trialB, which is necessary for potential subsequent randcluster statistics.
%
% Revision 1.3  2004/08/26 12:14:49  roboos
% *** empty log message ***
%
% Revision 1.2  2004/08/05 15:37:05  roboos
% added support for statistical parameters, fixed bug in leadfield
%
% Revision 1.1  2004/08/03 09:06:19  roboos
% initial implementation of these helper functions for beamformer sourceanalysis
%

if ~isfield(source, 'inside')  || ...
   ~isfield(source, 'outside') || ...
   ~isfield(source, 'xgrid')   || ...
   ~isfield(source, 'ygrid')   || ...
   ~isfield(source, 'zgrid')   || ...
   ~isfield(source, 'dim')
  error('one of the required fields is missing in the source structure');
end

% recreate the positions of the dipole grid
[X, Y, Z] = ndgrid(source.xgrid, source.ygrid, source.zgrid);
pos = [X(:) Y(:) Z(:)];

Nsparse = length(source.inside);
Nfull   = prod(source.dim);

% determine the size that each slice takes in memory
sx = 1;
sy = source.dim(1);
sz = source.dim(1) * source.dim(2);

% reconstruct the original inside and outside grid locations
for i=1:Nsparse
  fx = find(source.xgrid==source.pos(i,1));
  fy = find(source.ygrid==source.pos(i,2));
  fz = find(source.zgrid==source.pos(i,3));
  inside(i) = (fx-1)*sx + (fy-1)*sy + (fz-1)*sz + 1;
end
outside = setdiff(1:Nfull, inside);

fprintf('total number of dipoles        : %d\n', length(inside)+length(outside));
fprintf('number of dipoles inside  brain: %d\n', length(inside));
fprintf('number of dipoles outside brain: %d\n', length(outside));

% for each of the parameters, re-arrange the inside points
if isfield(source, 'prob'),      source.prob(inside)      = source.prob;      end
if isfield(source, 'stat'),      source.stat(inside)      = source.stat;      end
if isfield(source, 'ref'),       source.ref(inside)       = source.ref;       end
if isfield(source, 'tscore'),    source.tscore(inside)    = source.tscore;    end
if isfield(source, 'zscore'),    source.zscore(inside)    = source.zscore;    end
if isfield(source, 'sign'),      source.sign(inside)      = source.sign;      end
if isfield(source, 'mask'),      source.mask(inside)      = source.mask;      end
if isfield(source, 'leadfield'), source.leadfield(inside) = source.leadfield; end
if isfield(source, 'lbex'),      source.lbex(inside)      = source.lbex;      end

if isfield(source, 'avg')
  if isfield(source.avg, 'pow'),      source.avg.pow(inside)      = source.avg.pow;      end
  if isfield(source.avg, 'coh'),      source.avg.coh(inside)      = source.avg.coh;      end
  if isfield(source.avg, 'noise'),    source.avg.noise(inside)    = source.avg.noise;    end
  if isfield(source.avg, 'nai'),      source.avg.nai(inside)      = source.avg.nai;      end
  if isfield(source.avg, 'mom'),      source.avg.mom(inside)      = source.avg.mom;      end
  if isfield(source.avg, 'csd'),      source.avg.csd(inside)      = source.avg.csd;      end
  if isfield(source.avg, 'noisecsd'), source.avg.noisecsd(inside) = source.avg.noisecsd; end
end

if isfield(source, 'var')
  if isfield(source.var, 'pow'),   source.var.pow(inside)   = source.var.pow; end
  if isfield(source.var, 'coh'),   source.var.coh(inside)   = source.var.coh; end
  if isfield(source.var, 'noise'), source.var.noise(inside) = source.var.noise; end
  if isfield(source.var, 'nai'),   source.var.nai(inside)   = source.var.nai; end
  if isfield(source.var, 'mom'),   source.var.mom(inside)   = source.var.mom; end
end

if isfield(source, 'sem')
  if isfield(source.sem, 'pow'),   source.sem.pow(inside)   = source.sem.pow; end
  if isfield(source.sem, 'coh'),   source.sem.coh(inside)   = source.sem.coh; end
  if isfield(source.sem, 'noise'), source.sem.noise(inside) = source.sem.noise; end
  if isfield(source.sem, 'nai'),   source.sem.nai(inside)   = source.sem.nai; end
  if isfield(source.sem, 'mom'),   source.sem.mom(inside)   = source.sem.mom; end
end

if isfield(source, 'avgA')
  if isfield(source.avgA, 'pow'),   source.avgA.pow(inside)   = source.avgA.pow; end
  if isfield(source.avgA, 'coh'),   source.avgA.coh(inside)   = source.avgA.coh; end
  if isfield(source.avgA, 'noise'), source.avgA.noise(inside) = source.avgA.noise; end
  if isfield(source.avgA, 'nai'),   source.avgA.nai(inside)   = source.avgA.nai; end
  if isfield(source.avgA, 'mom'),   source.avgA.mom(inside)   = source.avgA.mom; end
end

if isfield(source, 'avgB')
  if isfield(source.avgB, 'pow'),   source.avgB.pow(inside)   = source.avgB.pow; end
  if isfield(source.avgB, 'coh'),   source.avgB.coh(inside)   = source.avgB.coh; end
  if isfield(source.avgB, 'noise'), source.avgB.noise(inside) = source.avgB.noise; end
  if isfield(source.avgB, 'nai'),   source.avgB.nai(inside)   = source.avgB.nai; end
  if isfield(source.avgB, 'mom'),   source.avgB.mom(inside)   = source.avgB.mom; end
end

if isfield(source, 'trialA')
  if isfield(source.trialA, 'pow'),   powA   = cat(1,source.trialA.pow);   powA(:,inside) = powA; end
  if isfield(source.trialA, 'coh'),   cohA   = cat(1,source.trialA.coh);   cohA(:,inside) = cohA; end
  if isfield(source.trialA, 'noise'), noiseA = cat(1,source.trialA.noise); noiseA(:,inside) = noiseA; end
  if isfield(source.trialA, 'nai'),   naiA   = cat(1,source.trialA.nai);   naiA(:,inside) = naiA; end
  if isfield(source.trialA, 'mom'),   momA   = cat(1,source.trialA.mom);   momA(:,inside) = momA; end
end

if isfield(source, 'trialB')
  if isfield(source.trialB, 'pow'),   powB   = cat(1,source.trialB.pow);   powB(:,inside) = powB; end
  if isfield(source.trialB, 'coh'),   cohB   = cat(1,source.trialB.coh);   cohB(:,inside) = cohB; end
  if isfield(source.trialB, 'noise'), noiseB = cat(1,source.trialB.noise); noiseB(:,inside) = noiseB; end
  if isfield(source.trialB, 'nai'),   naiB   = cat(1,source.trialB.nai);   naiB(:,inside) = naiB; end
  if isfield(source.trialB, 'mom'),   momB   = cat(1,source.trialB.mom);   momB(:,inside) = momB; end
end

% for each of the parameters, add the outside points
if isfield(source, 'prob'),      source.prob(outside)      = nan;        end
if isfield(source, 'stat'),      source.stat(outside)      = nan;        end
if isfield(source, 'ref'),       source.ref(outside)       = nan;        end
if isfield(source, 'tscore'),    source.tscore(outside)    = nan;        end
if isfield(source, 'zscore'),    source.zscore(outside)    = nan;        end
if isfield(source, 'sign'),      source.sign(outside)      = nan;        end
if isfield(source, 'mask'),      source.mask(outside)      = logical(0); end	% keep it logical if possible
if isfield(source, 'leadfield'), source.leadfield(outside) = {[]};       end
if isfield(source, 'lbex'),      source.lbex(outside)      = {[]};       end

if isfield(source, 'avg')
  if isfield(source.avg, 'pow'),      source.avg.pow(outside)      = nan;  end
  if isfield(source.avg, 'coh'),      source.avg.coh(outside)      = nan;  end
  if isfield(source.avg, 'noise'),    source.avg.noise(outside)    = nan;  end
  if isfield(source.avg, 'nai'),      source.avg.nai(outside)      = nan;  end
  if isfield(source.avg, 'mom'),      source.avg.mom(outside)      = {[]}; end
  if isfield(source.avg, 'csd'),      source.avg.csd(outside)      = {[]}; end
  if isfield(source.avg, 'noisecsd'), source.avg.noisecsd(outside) = {[]}; end
end

if isfield(source, 'var')
  if isfield(source.var, 'pow'),   source.var.pow(outside)   = nan; end
  if isfield(source.var, 'coh'),   source.var.coh(outside)   = nan; end
  if isfield(source.var, 'noise'), source.var.noise(outside) = nan; end
  if isfield(source.var, 'nai'),   source.var.nai(outside)   = nan; end
  if isfield(source.var, 'mom'),   source.var.mom(outside)   = {[]}; end
end

if isfield(source, 'sem')
  if isfield(source.sem, 'pow'),   source.sem.pow(outside)   = nan; end
  if isfield(source.sem, 'coh'),   source.sem.coh(outside)   = nan; end
  if isfield(source.sem, 'noise'), source.sem.noise(outside) = nan; end
  if isfield(source.sem, 'nai'),   source.sem.nai(outside)   = nan; end
  if isfield(source.sem, 'mom'),   source.sem.mom(outside)   = {[]}; end
end

if isfield(source, 'avgA')
  if isfield(source.avgA, 'pow'),   source.avgA.pow(outside)   = nan; end
  if isfield(source.avgA, 'coh'),   source.avgA.coh(outside)   = nan; end
  if isfield(source.avgA, 'noise'), source.avgA.noise(outside) = nan; end
  if isfield(source.avgA, 'nai'),   source.avgA.nai(outside)   = nan; end
  if isfield(source.avgA, 'mom'),   source.avgA.mom(outside)   = {[]}; end
end

if isfield(source, 'avgB')
  if isfield(source.avgB, 'pow'),   source.avgB.pow(outside)   = nan; end
  if isfield(source.avgB, 'coh'),   source.avgB.coh(outside)   = nan; end
  if isfield(source.avgB, 'noise'), source.avgB.noise(outside) = nan; end
  if isfield(source.avgB, 'nai'),   source.avgB.nai(outside)   = nan; end
  if isfield(source.avgB, 'mom'),   source.avgB.mom(outside)   = {[]}; end
end

if isfield(source, 'trialA')
  if isfield(source.trialA, 'pow'),   powA(:,outside) = nan;   source.trialA = fillfield(source.trialA,powA,'pow'); end
  if isfield(source.trialA, 'coh'),   cohA(:,outside) = nan;   source.trialA = fillfield(source.trialA,cohA,'coh');end
  if isfield(source.trialA, 'noise'), noiseA(:,outside) = nan; source.trialA = fillfield(source.trialA,noiseA,'noise');end
  if isfield(source.trialA, 'nai'),   naiA(:,outside) = nan;   source.trialA = fillfield(source.trialA,naiA,'nai');end
  %mom not yet supported
end

if isfield(source, 'trialB')
  if isfield(source.trialB, 'pow'),   powB(:,outside) = nan;   source.trialB = fillfield(source.trialB,powB,'pow');end
  if isfield(source.trialB, 'coh'),   cohB(:,outside) = nan;   source.trialB = fillfield(source.trialB,cohB,'coh');end
  if isfield(source.trialB, 'noise'), noiseB(:,outside) = nan; source.trialB = fillfield(source.trialB,noiseB,'noise');end
  if isfield(source.trialB, 'nai'),   naiB(:,outside) = nan;   source.trialB = fillfield(source.trialB,naiB,'nai');end
  %mom not  yet supported
end

% update the inside and outside definitions
source.inside  = inside;
source.outside = outside;
source.pos     = pos;

cfg = [];
% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: source2full.m,v 1.14 2005/09/09 17:08:21 jansch Exp $';
% remember the configuration details of the input data
try, cfg.previous = source.cfg; end
% remember the exact configuration details in the output 
source.cfg = cfg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function s = fillfield(s,matrix,parameter)
for j = 1:size(matrix,1)
  s(j) = setfield(s(j),parameter,matrix(j,:));
end
