function [source] = source2sparse(source);

% SOURCE2SPARCE removes the grid locations outside the brain from the source 
% reconstruction, thereby saving memory.
%
% This invalidates the fields that describe the grid, and also makes it
% more difficult to make a plot of each of the slices of the source volume.
% The original source structure can be recreated using SOURCE2FULL.
%
% Use as
%   [source] = source2sparse(source)
%
% See also SOURCE2FULL

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: source2sparse.m,v $
% Revision 1.7  2005/08/23 13:15:16  jansch
% added source.avg.csd and noisecsd
%
% Revision 1.6  2005/06/29 12:46:29  roboos
% the following changes were done on a large number of files at the same time, not all of them may apply to this specific file
% - added try-catch around the inclusion of input data configuration
% - moved cfg.version, cfg.previous and the assignment of the output cfg to the end
% - changed help comments around the configuration handling
% - some changes in whitespace
%
% Revision 1.5  2005/01/25 11:08:23  roboos
% added var and sem as fields containing sourceparameters
%
% Revision 1.4  2004/09/23 15:07:16  roboos
% added support for lbex as cell-array in the source structure
%
% Revision 1.3  2004/08/26 12:15:25  roboos
% added some extra fprintf information
%
% Revision 1.2  2004/08/05 15:37:05  roboos
% added support for statistical parameters, fixed bug in leadfield
%
% Revision 1.1  2004/08/03 09:06:19  roboos
% initial implementation of these helper functions for beamformer sourceanalysis
%

if ~isfield(source, 'inside')
  warning('no gridpoints defined inside the brain');
  source.inside = [];
end

if ~isfield(source, 'outside')
  warning('no gridpoints defined outside the brain');
  source.outside = [];
end

inside  = source.inside;
outside = source.outside;

fprintf('total number of dipoles        : %d\n', length(inside)+length(outside));
fprintf('number of dipoles inside  brain: %d\n', length(inside));
fprintf('number of dipoles outside brain: %d\n', length(outside));

% for each of the parameters, keep only the inside points
if isfield(source, 'prob'),      source.prob      = source.prob(inside);      end
if isfield(source, 'tscore'),    source.tscore    = source.tscore(inside);    end
if isfield(source, 'zscore'),    source.zscore    = source.zscore(inside);    end
if isfield(source, 'sign'),      source.sign      = source.sign(inside);      end
if isfield(source, 'mask'),      source.mask      = source.mask(inside);      end
if isfield(source, 'leadfield'), source.leadfield = source.leadfield(inside); end
if isfield(source, 'lbex'),      source.lbex      = source.lbex(inside);      end

if isfield(source, 'avg')
  if isfield(source.avg, 'pow'),      source.avg.pow      = source.avg.pow(inside);      end
  if isfield(source.avg, 'coh'),      source.avg.coh      = source.avg.coh(inside);      end
  if isfield(source.avg, 'noise'),    source.avg.noise    = source.avg.noise(inside);    end
  if isfield(source.avg, 'nai'),      source.avg.nai      = source.avg.nai(inside);      end
  if isfield(source.avg, 'mom'),      source.avg.mom      = source.avg.mom(inside);      end
  if isfield(source.avg, 'csd'),      source.avg.csd      = source.avg.csd(inside);      end
  if isfield(source.avg, 'noisecsd'), source.avg.noisecsd = source.avg.noisecsd(inside); end
end

if isfield(source, 'var')
  if isfield(source.var, 'pow'),   source.var.pow   = source.var.pow(inside); end
  if isfield(source.var, 'coh'),   source.var.coh   = source.var.coh(inside); end
  if isfield(source.var, 'noise'), source.var.noise = source.var.noise(inside); end
  if isfield(source.var, 'nai'),   source.var.nai   = source.var.nai(inside); end
  if isfield(source.var, 'mom'),   source.var.mom   = source.var.mom(inside); end
end

if isfield(source, 'sem')
  if isfield(source.sem, 'pow'),   source.sem.pow   = source.sem.pow(inside); end
  if isfield(source.sem, 'coh'),   source.sem.coh   = source.sem.coh(inside); end
  if isfield(source.sem, 'noise'), source.sem.noise = source.sem.noise(inside); end
  if isfield(source.sem, 'nai'),   source.sem.nai   = source.sem.nai(inside); end
  if isfield(source.sem, 'mom'),   source.sem.mom   = source.sem.mom(inside); end
end

if isfield(source, 'avgA')
  if isfield(source.avgA, 'pow'),   source.avgA.pow   = source.avgA.pow(inside); end
  if isfield(source.avgA, 'coh'),   source.avgA.coh   = source.avgA.coh(inside); end
  if isfield(source.avgA, 'noise'), source.avgA.noise = source.avgA.noise(inside); end
  if isfield(source.avgA, 'nai'),   source.avgA.nai   = source.avgA.nai(inside); end
  if isfield(source.avgA, 'mom'),   source.avgA.mom   = source.avgA.mom(inside); end
end

if isfield(source, 'avgB')
  if isfield(source.avgB, 'pow'),   source.avgB.pow   = source.avgB.pow(inside); end
  if isfield(source.avgB, 'coh'),   source.avgB.coh   = source.avgB.coh(inside); end
  if isfield(source.avgB, 'noise'), source.avgB.noise = source.avgB.noise(inside); end
  if isfield(source.avgB, 'nai'),   source.avgB.nai   = source.avgB.nai(inside); end
  if isfield(source.avgB, 'mom'),   source.avgB.mom   = source.avgB.mom(inside); end
end

% update the inside and outside definitions
source.inside  = 1:length(source.inside);
source.outside = [];
source.pos     = source.pos(inside,:);

cfg = [];
% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: source2sparse.m,v 1.7 2005/08/23 13:15:16 jansch Exp $';
% remember the configuration details of the input data
try, cfg.previous = source.cfg; end
% remember the exact configuration details in the output 
source.cfg = cfg;

