function sourcewrite(cfg, source)

% SOURCEWRITE exports source analysis results to an Analyze MRI file
% that can subsequently be read into BrainVoyager or MRIcro
%
% Use as
%   sourcewrite(cfg, source)
%   sourcewrite(cfg, stats)
%   sourcewrite(cfg, interp)
% where source contains the physical source reconstruction parameter
% (source), the statistical parameter (stat) or the interpolated and
% re-aligned anatomical MRI plus source reconstruction (interp).
%
% The structure cfg should contain the following elements
%   cfg.fileprefix  'string' filename without the extension (hdr/img)
%   cfg.parameter   'string' determines what should be written
%   cfg.datatype    'bit1', 'uint8', 'int16', 'int32', 'float' or 'double'
%   cfg.scaling     'yes' or 'no'
%
% By default, integer datatypes will be scaled to the maximum value of 
% the physical or statistical parameter. Floating point datatypes will 
% not be scaled.
%
% Optional configuration items are
%   cfg.fiducial.nas  [x y z]	position of nasion
%   cfg.fiducial.lpa  [x y z]	position of LPA
%   cfg.fiducial.rpa  [x y z]	position of RPA
%   
%   cfg.markfiducial  'yes' or 'no' 
%   cfg.markorigin    'yes' or 'no'
%   cfg.markcorner    'yes' or 'no' places marker at min(x,y,z)
%
% See also SOURCEANALYSIS, SOURCSTATISTICS

% Copyright (c) 2003 F.C. Donders Centre
%
% $Log: sourcewrite.m,v $
% Revision 1.8  2005/08/19 16:56:45  roboos
% add xgrid/ygrid/zgrid to the volume if not present
% swithched the selection of the functional volume to use the new subfunction parameterselection()
%
% Revision 1.7  2005/05/17 17:50:39  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.6  2005/05/04 07:33:37  roboos
% made chaling optional for int datatypes, added cfg.scaling option
%
% Revision 1.5  2004/10/14 09:33:11  roboos
% added nai, prob and mask as parameters
% improved documentation
%
% Revision 1.4  2003/12/18 22:21:35  roberto
% added "See also" line to the online help
%
% Revision 1.3  2003/12/08 12:33:47  roberto
% only layout changes in code
%
% Revision 1.2  2003/10/28 15:12:04  roberto
% fixed bug: switch...otherwise (instead of switch...default)
%
% Revision 1.1  2003/07/23 09:11:15  roberto
% fixed bug in integer scaling
%

% set the defaults
if ~isfield(cfg, 'markorigin')   cfg.markorigin   = 'no'; end
if ~isfield(cfg, 'markfiducial') cfg.markfiducial = 'no'; end
if ~isfield(cfg, 'markcorner')   cfg.markcorner   = 'no'; end
if ~isfield(cfg, 'datatype')     cfg.datatype     = 'int16'; end
if ~isfield(cfg, 'scaling'),
  if any(strmatch(cfg.datatype, {'int8', 'int16', 'int32'}))
    cfg.scaling = 'yes';
  else
    cfg.scaling = 'no';
  end
end

% for backward compatibility
if ~isfield(source, 'xgrid'), source.xgrid=1:source.dim(1); end
if ~isfield(source, 'ygrid'), source.ygrid=1:source.dim(2); end
if ~isfield(source, 'zgrid'), source.zgrid=1:source.dim(3); end

% select the parameter that should be written
cfg.parameter = parameterselection(cfg.parameter, source);

% convert the data into double values so that it can be scaled later
data = double(getsubfield(source, cfg.parameter));

x = source.xgrid;
y = source.ygrid;
z = source.zgrid;
minxyz = [min(x) min(y) min(z)];
maxxyz = [max(x) max(y) max(z)];
siz = [length(x) length(y) length(z)];
maxval = max(data(:));

% ensure that the volume data is 3D
data = reshape(data(:), siz);

if strcmp(cfg.markfiducial, 'yes')
  % determine the voxel index of the fiducials
  nas = cfg.fiducial.nas;
  lpa = cfg.fiducial.lpa;
  rpa = cfg.fiducial.rpa;
  if any(nas<minxyz) || any(nas>maxxyz) 
    warning('nasion does not ly within volume, using nearest voxel');
  end
  if any(lpa<minxyz) || any(lpa>maxxyz) 
    warning('LPA does not ly within volume, using nearest voxel');
  end
  if any(rpa<minxyz) || any(rpa>maxxyz) 
    warning('RPA does not ly within volume, using nearest voxel');
  end
  idx_nas = [nearest(x, nas(1)) nearest(y, nas(2)) nearest(z, nas(3))];
  idx_lpa = [nearest(x, lpa(1)) nearest(y, lpa(2)) nearest(z, lpa(3))];
  idx_rpa = [nearest(x, rpa(1)) nearest(y, rpa(2)) nearest(z, rpa(3))];
  fprintf('NAS corresponds to voxel [%d, %d, %d]\n', idx_nas);
  fprintf('LPA corresponds to voxel [%d, %d, %d]\n', idx_lpa);
  fprintf('RPA corresponds to voxel [%d, %d, %d]\n', idx_rpa);
  data(idx_nas(1), idx_nas(2), idx_nas(3)) = maxval;
  data(idx_lpa(1), idx_lpa(2), idx_lpa(3)) = maxval;
  data(idx_rpa(1), idx_rpa(2), idx_rpa(3)) = maxval;
end

if strcmp(cfg.markorigin, 'yes')
  % determine the voxel index of the coordinate system origin
  ori = [0 0 0];
  if any(ori<minxyz) || any(ori>maxxyz)
    warning('origin does not ly within volume, using nearest voxel');
  end
  idx_ori = [nearest(x, ori(1)) nearest(y, ori(2)) nearest(z, ori(3))];
  fprintf('origin corresponds to voxel [%d, %d, %d]\n', idx_ori);
  data(idx_ori(1), idx_ori(2), idx_ori(3)) = maxval;
end

if strcmp(cfg.markcorner, 'yes')
  % add a marker to the posterior-right-inferior corner
  data(1:2, 1:1, 1:1) = maxval;		% length 2 along x-axis
  data(1:1, 1:3, 1:1) = maxval;		% length 3 along y-axis
  data(1:1, 1:1, 1:4) = maxval;		% length 4 along z-axis
end

% remove not-a-number voxels
data(isnan(data)) = 0;

% create an empty Analyze header structure
avw = avw_hdr_make;

% The coordinate system employed by the ANALYZE programs is left-handed, 
% with the coordinate origin in the lower left corner. Thus, with the 
% subject lying supine, the coordinate origin is on the right side of 
% the body (x), at the back (y), and at the feet (z).

% orientation 0 means transverse unflipped (axial, radiological)
% X direction first,  progressing from patient right to left, 
% Y direction second, progressing from patient posterior to anterior, 
% Z direction third,  progressing from patient inferior to superior. 
avw.hdr.hist.orient = 0;

% reorganize the dimensions of the volume to radiological Analyze convention
% CTF x = post-ant	    analyze x = right-left
% CTF y = right-left        analyze y = post-ant
% CTF z = inf-sup           analyze z = inf-sup
avw.img = permute(data, [2 1 3]);

% For the 'transverse unflipped' type, the voxels are stored as transaxial
% slices with the 'x' direction first (varies fastest), which is oriented such
% that it progresses from patient right to patient left; followed by the 'y'
% axis, which goes from patient posterior to anterior; followed by the
% 'z' axis, which is the slice direction and moves from inferior to
% superior.  When data is loaded by the Analyze software, this is the
% default order of voxels and anatomic orientation in the coordinate
% system (left-handed coordinate system with the origin in the right,
% posterior, inferior position relative to the body).

% adjust the image dimensions 
avw.hdr.dime.dim(2:4) = size(avw.img);

% adjust voxel size, changing from CTF coordinates to Analyze
resx = x(2)-x(1);
resy = y(2)-y(1);
resz = z(2)-z(1);
avw.hdr.dime.pixdim(2:4) = [resy resx resz];

% modify the data type
switch lower(cfg.datatype)
case 'bit1'
  avw.hdr.dime.datatype = 1;
  avw.hdr.dime.bitpix   = 1;
  avw.img = (avg.img~=0);
case 'uint8'
  avw.hdr.dime.datatype = 2;
  avw.hdr.dime.bitpix   = 8;
  if strcmp(cfg.scaling, 'yes')
    fprintf('scaling data to full uint8 data range\n');
    avw.img = uint8((2^8-1) * avw.img/maxval);
  end
case 'int16'
  avw.hdr.dime.datatype = 4;
  avw.hdr.dime.bitpix   = 16;
  if strcmp(cfg.scaling, 'yes')
    fprintf('scaling data to full int16 data range\n');
    avw.img = int16((2^15-1) * avw.img/maxval);
  end
case 'int32'
  avw.hdr.dime.datatype = 8;
  avw.hdr.dime.bitpix   = 32;
  if strcmp(cfg.scaling, 'yes')
    fprintf('scaling data to full int32 data range\n');
    avw.img = int32((2^31-1) * avw.img/maxval);
  end
case 'float'
  avw.hdr.dime.datatype = 16;
  avw.hdr.dime.bitpix   = 32;
  % no scaling neccessary for float
  if strcmp(cfg.scaling, 'yes')
    warning('no scaling is performed for float data type');
  end
case 'double'
  avw.hdr.dime.datatype = 64;
  avw.hdr.dime.bitpix   = 64;
  % no scaling neccessary for double
  if strcmp(cfg.scaling, 'yes')
    warning('no scaling is performed for double data type');
  end
otherwise
  error('unknown datatype');
end

% write the header and image data
avw_img_write(avw, cfg.fileprefix, ' ', 'ieee-le');

