function topoplotER(cfg, data)                                               

% TOPOPLOTER makes topographic plots of event-related fields or potentials
% or of the scalp distribution of oscillatory activity (power or coherence).
%
% Use as
%   topoplotER(cfg, data)
%
% The data can be an ERP/ERF produced by TIMELOCKANALYSIS, a
% powerspectrum produced by FREQANALYSIS or a coherencespectrum produced
% by FREQDESCRIPTIVES. If you specify multiple datasets they must contain
% the same channels, etc.
%
% The cfg structure contains all configurations details for the plotting
%
% cfg.layout       = specification of the layout, see below
% cfg.xparam       = field defining the x-range (default = 'time')
% cfg.zparam       = field defining the color, i.e. z-range (default = 'avg')
% cfg.xlim         = [time1 time2] (default = full interval)
%                    Time interval for which to plot 
%                  = [time1 time2 time3 ... timeN] 
%                    Do N-1 plots: [time1 time2],[time2 time3] ... 
% cfg.zlim         = 'maxmin','absmax' or [zmin zmax] (default = 'absmax')
%                    Set the 'color limits'
% cfg.baseline     = 'yes','no' or [time1 time2] (default = 'no')  
% cfg.baselinetype = 'absolute','relative' (default = 'absolute')  
% cfg.comment      = string of text (default = date + colors)
%                    Add 'comment' to graph (according to COMNT in the layout)
% cfg.showlabels   = 'yes','no','markers','numbers' (default = 'no')
%                    Plot sensors labels/numbers or markers at each graph  
% cfg.colorbar     = 'yes','no' (default = 'yes')            
% cfg.showxlim     = 'yes','no' (default = 'yes')
% cfg.showzlim     = 'yes','no' (default = 'yes')            
%                    Plot the color limits at the bottom 
% cfg.fontsize     =  size of font (default = 8)             
% 
% If you use the output of freqdescriptives as an input and if you want to visualise
% coherence with respect to a reference-channel, you should specify:
% 
% cfg.zparam        =  'cohspctrm'
% cfg.cohrefchannel =  name of reference-channel
%
% The layout defines what the position of each channel is. These
% positions are used for the interpolation of the field or potential.
% You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure, that will be used for creating a
% layout. If you want to have more fine-grained control over the layout,
% you should create your own layout file.
% 
% See also TOPOPLOTFTR, MULTIPLOTER, MULTIPLOTTFR

% Copyrights (C) 2003, Ole Jensen
%
% $Log: topoplotER.m,v $
% Revision 1.21  2005/08/30 11:26:23  jansch
% included explicit error if the specified latency-window or frequency-window
% are not present in the data.
%
% Revision 1.20  2005/08/19 07:50:07  jansch
% included hidden-option cfg.subplot to gain some flexibility in the amount
% of subplots per row and column.
%
% Revision 1.19  2005/08/11 15:12:09  jansch
% included the possibility to plot the reference-marker along, when plotting
% cortico-cortical coherence
%
% Revision 1.18  2005/08/08 10:13:05  jansch
% included the option to plot z-transformed coherence
%
% Revision 1.17  2005/08/08 09:43:36  jansch
% changed getfield into getsubfield, so that it can deal with nested strutures
% as zparam.
%
% Revision 1.16  2005/07/03 20:15:48  roboos
% replaced eval() by getfield() for better support by Matlab compiler
%
% Revision 1.15  2005/06/06 08:05:59  roboos
% fixed bug in baseline for timelocked data (replaced toi by xvalues)
%
% Revision 1.14  2005/05/23 14:03:38  roboos
% added default for showzlim
%
% Revision 1.13  2005/05/17 17:50:40  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.12  2005/04/29 13:04:36  roboos
% changed yparam into zparam for consistency
% updated the help
%
% Revision 1.11  2005/04/22 07:46:10  roboos
% removed empty line befor help block that was causing difficulties in the awk parsing of the website help
%
% Revision 1.10  2005/04/13 09:00:14  roboos
% added two lines with comments about rouding-off error in tidx
%
% Revision 1.9  2005/04/05 08:37:09  jansch
% *** empty log message ***
%
% Revision 1.8  2005/04/05 07:52:07  jansch
% fixed small bug
%
% Revision 1.7  2005/04/05 07:48:27  jansch
% included the option to visualise coherence. updated help
%
% Revision 1.6  2005/02/08 08:32:52  roboos
% fixed bug that was introduced in the last change (varargin->data)
%
% Revision 1.5  2005/02/07 17:33:55  roboos
% implemented support for new createlayout function for automatic construction of layout absed on gradiometer definitions that are present in the data, updated help and improved layout of the help
%
% Revision 1.4  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%

% for backward support with old configurations
if isfield(cfg, 'yparam') && ~isfield(cfg, 'zparam')
  warning('please specify cfg.zparam instead of cfg.yparam');
  cfg.zparam = cfg.yparam;
  cfg = rmfield(cfg, 'yparam');
end

% set the defaults
if ~isfield(cfg,'baseline'),        cfg.baseline     = 'no';        end;
if ~isfield(cfg,'baselinetype'),    cfg.baselinetype = 'absolute';  end;
if ~isfield(cfg,'zlim'),            cfg.zlim         = 'absmax';    end;
if ~isfield(cfg,'comment'),         cfg.comment      = strcat([date '\n' ]); end;
if ~isfield(cfg,'showlabels'),      cfg.showlabels   = 'no';        end;
if ~isfield(cfg,'showxlim'),        cfg.showxlim     = 'yes';       end;
if ~isfield(cfg,'showzlim'),        cfg.showzlim     = 'yes';       end;
if ~isfield(cfg,'colorbar'),        cfg.colorbar     = 'yes';       end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize     = 8;           end;
if ~isfield(cfg,'xparam'),          cfg.xparam       = 'time';      end;
if ~isfield(cfg,'zparam'),          cfg.zparam       = 'avg';       end;
if ~isfield(cfg,'subplot'),         cfg.subplot      = 'default';   end;

if any(strcmp(cfg.zparam, {'cohspctrm' 'avg.cohspctrm' 'avg.zcohspctrm' 'avg.icohspctrm'})),
  if ~isfield(cfg,'cohrefchannel'),
    error('no reference channel specified');
  else
    refindx = strmatch(cfg.cohrefchannel, data.label);
    if isempty(refindx),
      error('reference channel not present in data');
    end
  end
end

GRAPHCOLOR =['brgbkywrgbkywrgbkywrgbkyw'];

% get the data that should be plotted
P       = getsubfield(data, cfg.zparam);
xvalues = getsubfield(data, cfg.xparam);

% set the default for this parameter based on the data
if ~isfield(cfg,'xlim'),
  cfg.xlim  = [min(xvalues) max(xvalues)];
end

% xlim always needs a specification like [begin end], even if you want to plot a single bin
if all(size(cfg.xlim)==[1 1])
  cfg.xlim = [cfg.xlim cfg.xlim];
end

if any(strcmp(cfg.zparam, {'cohspctrm' 'avg.cohspctrm' 'avg.zcohspctrm' 'avg.icohspctrm'}))
  sel1       = strmatch(cfg.cohrefchannel, data.labelcmb(:,2));
  sel2       = strmatch(cfg.cohrefchannel, data.labelcmb(:,1));
  P          = [P(sel1,:); zeros(1,size(P,2)); P(sel2,:)];
  data.label = [data.labelcmb(sel1,1);data.label(refindx);data.labelcmb(sel2,2)];
end

% read or create the layout that will be used for plotting
if isfield(cfg, 'layout')
  lay = createlayout(cfg.layout);               % use specified layout
elseif isfield(cfg, 'layoutname')
  lay = createlayout(cfg.layoutname);           % idem, for backward compatibility
elseif isfield(data, 'grad')
  lay = createlayout(data.grad);                % create layout from gradiometer definition
elseif isfield(data, 'elec')
  lay = createlayout(data.elec);                % create layout from electrode definition
else
  lay = createlayout('CTF151s.lay');            % revert to the FCDC default
end

% convert the layout to Ole's style of variable names
X      = lay.prj(:,1);
Y      = lay.prj(:,2);
Width  = lay.box(:,1);
Height = lay.box(:,2);
Lbl    = lay.label;

if strcmp(cfg.baseline,'yes')
  % default is to use pretrigger interval for baseline
  cfg.baseline = [];
  cfg.baseline(1) = min(xvalues);
  cfg.baseline(2) = 0;
end

% compute and apply the baseline correction
if ~isempty(cfg.baseline) && ~strcmp(cfg.baseline, 'no')
  tidx = find(xvalues >= cfg.baseline(1) & xvalues <= cfg.baseline(2)); 
  if ~isempty(tidx)
    tmp = mean(P(:,tidx),2);
    for l=1:size(P,1)
      if strcmp(cfg.baselinetype,'absolute')
        P(l,:) = P(l,:) - tmp(l);
      elseif strcmp(cfg.baselinetype,'relative')
        P(l,:) = 100*(P(l,:) - tmp(l))./tmp(l);
      end
    end
  end
end

% find matching channels in the data and the layout
[sensorOrder, dataOrder] = match_str(Lbl, data.label);
fprintf('selected %d channels for plotting\n', length(sensorOrder));

% this loop creates multiple subplots in a row, each with a different
% timeselection
for k=1:length(cfg.xlim)-1 
  if length(cfg.xlim)-1 > 1 
    if isstr(cfg.subplot) && strcmp(cfg.subplot, 'default'),
      subplot(ceil((length(cfg.xlim)-1)/6),6,k)
    else
      spx = cfg.subplot(2);
      spy = cfg.subplot(1);
      nsp = length(cfg.xlim)-1;
      if mod(k-1, spx*spy)==0,
        figure;
      end
      subplot(spy, spx, rem(k-1, spx*spy)+1);
    end
  end
  %tidx = find(cfg.xlim(k) <= xvalues & (cfg.xlim(k+1) > xvalues));
  %THIS GIVES AN EMPTY VECTOR IF XLIM(1)==XLIM(2)!!!!!!
  tidx = find(cfg.xlim(k) <= xvalues & (cfg.xlim(k+1) >= xvalues));
  % Vladimir reported rounding-off errors, maybe the following line would fix it
  % tidx = nearest(xvalues, cfg.xlim(k)): nearest(xvalues, cfg.xlim(k+1));
  if length(tidx)>1
      V = mean(P(:,tidx),2);
  elseif length(tidx)==1
      V = P(:,tidx);
  elseif length(tidx)==0
      error('there is no data selected to be plotted');
  end;    
  topoplot(cfg,X(sensorOrder),Y(sensorOrder),V(dataOrder),Lbl(sensorOrder))
  if strcmp(cfg.showxlim,'yes')
    title(sprintf('Time:\n%.2e to %.2e ',cfg.xlim(k),cfg.xlim(k+1)),'fontsize',cfg.fontsize)
  end
  drawnow
end
