function topoplotTFR(cfg,data)                                               

% TOPOPLOTTFR plots the topographic distribution of the spectrum or
% the time-frequence representations of power or coherence that were
% computed using the FREQANALYSIS or FREQDESCRIPTIVES functions.
%
% Use as
%   topoplotTFR(cfg, freq)
% where freq is the output of FREQANALYSIS, eventually after combining
% the TFRs of the planar gradients using COMBINEPLANAR. The cfg
% structure contains all configuration details for the plotting.
%
% cfg.layout       = specification of the layout, see below
% cfg.baseline     = 'yes','no' or [time1 time2] (default = 'no')  
% cfg.baselinetype = 'absolute','relative' (default = 'absolute')  
% cfg.xlim         = [time1 time2] (default = full interval)
%                    Time interval for which to plot 
%                  = [time1 time2 time3 ... timeN] 
%                    Do N-1 plots: [time1 time2],[time2 time3] ... 
% cfg.ylim         = [freq1 freq2] (default = full interval)  
%                    Frequency interval for which to plot
% cfg.zlim         = 'maxmin','absmax' or [ymin ymax] (default = 'absmax')
%                    Set the 'color limits'
% cfg.showlabels   = 'yes','no','markers','numbers' (default = 'no')         
%                     Plot sensors labels/numbers or markers at each graph  
% cfg.colorbar     = 'yes','no' (default = 'yes')            
% cfg.showzlim     = 'yes','no' (default = 'yes')            
%                    Plot the color limits at the bottom 
% cfg.showxlim     = 'yes','no' (default = 'yes')
% cfg.fontsize     =  size of font (default = 8)             
% cfg.yparam       = 'powspctrm' (default) or 'cohspctrm'
% 
% In case cfg.yparam   = 'cohspctrm', you should specify the reference-channel in
% cfg.cohrefchannel    = string
%
% The layout defines what the position of each channel is. These
% positions are used for the interpolation of the field or potential.
% You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure (common for MEG data, since the header
% of the MEG datafile contains the gradiometer information), that will be
% used for creating a layout. If you want to have more fine-grained control
% over the layout, you should create your own layout file.
% 
% See also TOPOPLOTER, MULTIPLOTTFR, MULTIPLOTER

% Copyright (C) 2003, Ole Jensen
%
% $Log: topoplotTFR.m,v $
% Revision 1.9  2005/05/17 17:50:40  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.8  2005/04/08 17:30:53  olejen
% TFR was wrongly assigned before calling freqbaseline; this is now corrected
%
% Revision 1.7  2005/04/06 14:31:27  olejen
% now use freqbaseline.m
%
% Revision 1.6  2005/04/06 07:40:56  jansch
% included option cfg.cohrefchannel. updated help.
%
% Revision 1.5  2005/02/21 07:25:05  roboos
% fixed bug (replaced varargin by data)
%
% Revision 1.4  2005/02/07 17:33:55  roboos
% implemented support for new createlayout function for automatic construction of layout absed on gradiometer definitions that are present in the data, updated help and improved layout of the help
%
% Revision 1.3  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%

if ~isfield(data, 'toi')
  fprintf('looks like power spectrum with no time information\n');
  % add a virtual time axis
  data.toi = 0;
end

% set the defaults
if ~isfield(cfg,'baseline'),        cfg.baseline     = 'no';                   end;
if ~isfield(cfg,'baselinetype'),    cfg.baselinetype = 'absolute';             end;
if ~isfield(cfg,'xlim'),            cfg.xlim  = [min(data.toi) max(data.toi)]; end;
if ~isfield(cfg,'ylim'),            cfg.ylim  = [min(data.foi) max(data.foi)]; end;
if ~isfield(cfg,'zlim'),            cfg.zlim       = 'absmax';                 end;
if ~isfield(cfg,'comment'),         cfg.comment    = strcat([date '\n' ]);     end;
if ~isfield(cfg,'showlabels'),      cfg.showlabels = 'no';                     end;
if ~isfield(cfg,'showxlim'),        cfg.showxlim   = 'yes';                    end;
if ~isfield(cfg,'showzlim'),        cfg.showzlim   = 'yes';                    end;
if ~isfield(cfg,'colorbar'),        cfg.colorbar   = 'yes';                    end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize   = 8;                        end;
if ~isfield(cfg,'yparam'),          cfg.yparam     = 'powspctrm';              end;
if strcmp(cfg.yparam,'cohspctrm') && ~isfield(cfg,'cohrefchannel'),
  error('no reference channel specified');
end

% it always needs a specification like [begin end], even if you want to plot a single bin
if all(size(cfg.xlim)==[1 1])
  cfg.xlim = [cfg.xlim cfg.xlim];
end
if all(size(cfg.ylim)==[1 1])
  cfg.ylim = [cfg.ylim cfg.ylim];
end

GRAPHCOLOR =['brgbkywrgbkywrgbkywrgbkyw'];



if strcmp(cfg.baseline,'yes')
  % default is to use pretrigger interval for baseline
  cfg.baseline = [];
  cfg.baseline(1) = min(data.toi);
  cfg.baseline(2) = 0;
end


if ~isempty(cfg.baseline) && ~strcmp(cfg.baseline, 'no')
  % compute the baselin  
  data = freqbaseline(cfg,data);
end

if strcmp(cfg.yparam,'powspctrm');
  TFR        = data.powspctrm;
elseif strcmp(cfg.yparam,'cohspctrm');
  sel1       = strmatch(cfg.cohrefchannel, data.labelcmb(:,2));
  sel2       = strmatch(cfg.cohrefchannel, data.labelcmb(:,1));
  TFR        = data.cohspctrm([sel1;sel2],:,:);
  data.label = [data.labelcmb(sel1,1);data.labelcmb(sel2,2)]; 
end

% read or create the layout that will be used for plotting
if isfield(cfg, 'layout')
  lay = createlayout(cfg.layout);               % use specified layout
elseif isfield(cfg, 'layoutname')
  lay = createlayout(cfg.layoutname);           % idem, for backward compatibility
elseif isfield(data, 'grad')
  lay = createlayout(data.grad);         % create layout from gradiometer definition
elseif isfield(data, 'elec')
  lay = createlayout(data.elec);         % create layout from electrode definition
else
  lay = createlayout('CTF151s.lay');            % revert to the FCDC default
end

% convert the layout to Ole's style of variable names
X      = lay.prj(:,1);
Y      = lay.prj(:,2);
Width  = lay.box(:,1);
Height = lay.box(:,2);
Lbl    = lay.label;



% find matching channels in the data and the layout
[sensorOrder, dataOrder] = match_str(Lbl, data.label);
fprintf('selected %d channels for plotting\n', length(sensorOrder));

% this loop creates multiple subplots in a row, each with a different
% timeselection
for k=1:length(cfg.xlim)-1 
  subplot(1,length(cfg.xlim)-1,k)
  % select the time bins of interest
  xidc = find(data.toi >= cfg.xlim(k) & data.toi <= cfg.xlim(k+1));
  % select the frequency bins of interest
  yidc = find(data.foi >= cfg.ylim(1) & data.foi <= cfg.ylim(2));
  fprintf('averaging over %d time bins and %d frequency bins\n', length(xidc), length(yidc));
  
  % compute the mean topography over selected time-frequency window and plot it
  TFRval = nan_mean(nan_mean(TFR(:,yidc,xidc),2),3);
  topoplot(cfg,X(sensorOrder),Y(sensorOrder),TFRval(dataOrder),Lbl(sensorOrder))
  
  % add a label for the timewindow for this specific subplot
  if strcmp(cfg.showxlim,'yes')
    title(sprintf('Time:\n%.2e to %.2e ',cfg.xlim(k),cfg.xlim(k+1)),'fontsize',cfg.fontsize)
  end
  drawnow
end
